# pylint: disable=missing-docstring,invalid-name,protected-access,too-few-public-methods
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import time
import unittest
import logging
import mock

from icetea_lib.TestBench.Commands import Commands
from icetea_lib.CliAsyncResponse import CliAsyncResponse
from icetea_lib.TestStepError import TestStepFail, TestStepTimeout, TestStepError


class MockLogger(object):
    def __init__(self):
        pass

    def info(self, *args, **kwargs):
        pass

    def debug(self, *args, **kwargs):
        pass

    def error(self, *args, **kwargs):
        pass

    def warning(self, *args, **kwargs):
        pass


class MockResponse(object):
    def __init__(self, lines=None, retcode=0):
        self.lines = lines if lines else []
        self.retcode = retcode
        self.timeout = None


class MockReq(object):
    def __init__(self, response=None):
        self.response = response if response else None
        self.dut_index = 1
        self.cmd = "test_command"


class CommandMixerTestcase(unittest.TestCase):

    def test_command_fail_nameerror(self):
        cmixer = Commands(mock.MagicMock(), mock.MagicMock(), mock.MagicMock(),
                          mock.MagicMock(), mock.MagicMock())
        cmixer._logger = MockLogger()
        with self.assertRaises(NameError):
            cmixer._command_fail(None, fail_reason="this_fail")

    def test_command_fail(self):
        response = MockResponse(["line1", "line2"], 1001)
        request = MockReq(response)
        cmixer = Commands(mock.MagicMock(), mock.MagicMock(), mock.MagicMock(),
                          mock.MagicMock(), mock.MagicMock())
        cmixer._logger = MockLogger()

        with self.assertRaises(TestStepFail):
            cmixer._command_fail(request)
        with self.assertRaises(TestStepFail):
            request.response.retcode = -5
            cmixer._command_fail(request)
        with self.assertRaises(TestStepFail):
            request.response.retcode = -2
            cmixer._command_fail(request)
        with self.assertRaises(TestStepFail):
            request.response.retcode = -3
            cmixer._command_fail(request)
        with self.assertRaises(TestStepFail):
            request.response.retcode = -4
            cmixer._command_fail(request)
        with self.assertRaises(TestStepTimeout):
            request.response.timeout = True
            cmixer._command_fail(request)

        request.response = None
        with self.assertRaises(TestStepFail):
            cmixer._command_fail(request)

    def test_wait_for_async_response_attribute_error(self):
        cmixer = Commands(mock.MagicMock(), mock.MagicMock(), mock.MagicMock(),
                          mock.MagicMock(), mock.MagicMock())
        cmixer._logger = MockLogger()
        with self.assertRaises(AttributeError):
            cmixer.wait_for_async_response("test_command", None)

    def test_wait_for_async_response(self):
        mocked_plugins = mock.MagicMock()
        cmixer = Commands(mock.MagicMock(), mocked_plugins, mock.MagicMock(),
                          mock.MagicMock(), mock.MagicMock())
        cmixer._logger = MockLogger()
        mocked_dut = mock.MagicMock()
        async_resp = CliAsyncResponse(mocked_dut)
        response = MockResponse(["line1", "line2"], 1001)
        async_resp.parsed = True
        async_resp.response = response
        self.assertEqual(response, cmixer.wait_for_async_response("test_cmd", async_resp))

        async_resp.parsed = False
        mocked_plugins.parse_response = mock.MagicMock(return_value={"parsed": "resp"})
        retval = cmixer.wait_for_async_response("test_command", async_resp)
        self.assertDictEqual(retval.parsed, {"parsed": "resp"})
        self.assertEqual(response, retval)

    def test_execute_command_execute_failures(self):
        cmixer = Commands(mock.MagicMock(), mock.MagicMock(), mock.MagicMock(),
                          mock.MagicMock(), mock.MagicMock())
        cmixer._logger = MockLogger()
        cmixer.get_time = time.time
        mocked_dut = mock.MagicMock()
        mocked_dut.execute_command = mock.MagicMock()
        mocked_dut.execute_command.side_effect = [TestStepFail, TestStepError, TestStepTimeout]

        with self.assertRaises(TestStepFail):
            cmixer._execute_command(mocked_dut, "test_command")
        with self.assertRaises(TestStepError):
            cmixer._execute_command(mocked_dut, "test_command")
        with self.assertRaises(TestStepTimeout):
            cmixer._execute_command(mocked_dut, "test_command")

    def test_private_execute_command(self):
        cmixer = Commands(mock.MagicMock(), mock.MagicMock(), mock.MagicMock(),
                          mock.MagicMock(), mock.MagicMock())
        cmixer._logger = MockLogger()
        cmixer.get_time = time.time
        mocked_dut = mock.MagicMock()
        mocked_dut.execute_command = mock.MagicMock()
        response = MockResponse(["line1"], retcode=0)
        mocked_dut.execute_command.return_value = response
        cmixer._parse_response = mock.MagicMock(return_value={"test": "parsed"})
        self.assertEqual(cmixer._execute_command(mocked_dut, "test_command"), response)

    def test_sync_cli(self):
        mock_gen = mock.MagicMock(return_value=("val1", "val2"))
        mocked_resources = mock.MagicMock()
        logger = logging.getLogger('unittest')
        logger.addHandler(logging.NullHandler())
        cmds = Commands(mock.MagicMock(), mock.MagicMock(), mocked_resources,
                        mock.MagicMock(), mock.MagicMock())
        cmds._logger = logger
        mocked_dut = mock.MagicMock()
        mocked_resources.get_dut = mock.MagicMock(return_value=mocked_dut)
        type(mocked_dut).config = mock.PropertyMock(return_value=dict())
        with self.assertRaises(TestStepError):
            with mock.patch.object(cmds, "execute_command"):
                cmds.sync_cli("1", mock_gen, retries=1)

    @mock.patch("icetea_lib.TestBench.Commands.uuid")
    def test_echo_uuid_generator(self, mock_uuid):
        mock_uuid.uuid1 = mock.MagicMock(return_value="uuid")
        tpl = Commands.get_echo_uuid()
        expected_cmd = "echo uuid"
        expected_retval = "uuid"
        self.assertEqual(tpl[0], expected_cmd)
        self.assertEqual(tpl[1], expected_retval)


if __name__ == '__main__':
    unittest.main()



# pylint: disable=missing-docstring,invalid-name

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import os
import unittest

from jsonschema import validate, ValidationError


class ValidatorTestcase(unittest.TestCase):

    def setUp(self):
        with open(os.path.abspath(os.path.join(os.path.dirname(__file__),
                                               "..", "icetea_lib",
                                               'tc_schema.json'))) as data_file:
            self.tc_meta_schema = json.load(data_file)

    def test_validation_successful(self):
        meta = {"name": "test_case",
                "title": "test_case",
                "status": "development",
                "purpose": "unittest",
                "component": ["validator"],
                "type": "smoke",
                "requirements": {
                    "duts": {
                        "*": {
                            "count": 1,
                            "type": "process",
                            "platform_name": "test_plat",
                            "application": {
                                "version": "1.0.0",
                                "name": "app_name",
                                "bin": "binary",
                                "cli_ready_trigger": "trigger",
                                "cli_ready_trigger_timeout": 10,
                                "init_cli_cmds": [],
                                "post_cli_cmds": [],
                                "bin_args": ["arg1", "arg2"]
                                },
                            "location": [],
                            }
                        }
                    },
                "external": {
                    "ExtApp": "ExtAppName"
                    }
               }
        validate(meta, self.tc_meta_schema)

    def test_validation_successful_with_extra_fields_and_missing_fields(self):
        meta = {"extra_field": {"This field does not exist in the schema": "data"}}

        validate(meta, self.tc_meta_schema)

    def test_validation_causes_error(self):
        meta = {"name": "test_case",
                "title": "test_case",
                "status": "development",
                "purpose": "unittest",
                "component": "validator",  # This should cause an error
                "type": "smoke",
                "requirements": {
                    "duts": {
                        "*": {
                            "count": 1,
                            "type": "process",
                            "platform_name": "test_plat",
                            "application": {
                                "version": "1.0.0",
                                "name": "app_name",
                                "bin": "binary",
                                "cli_ready_trigger": "trigger",
                                "cli_ready_trigger_timeout": 10,
                                "init_cli_cmds": [],
                                "post_cli_cmds": [],
                                "bin_args": ["arg1", "arg2"]
                                },
                            "location": [0.0, 0.0],
                            }
                        }
                    },
                "external": {
                    "ExtApp": "ExtAppName"
                }
               }
        with self.assertRaises(ValidationError):
            validate(meta, self.tc_meta_schema)

        meta["component"] = ["validator"]
        meta["status"] = "Not_exists" # This should cause an error

        with self.assertRaises(ValidationError):
            validate(meta, self.tc_meta_schema)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

CliAsyncResponse module, contains CliAsyncResponse class,
which is a proxy for a future CliResponse.

"""

import icetea_lib.LogManager as LogManager


class CliAsyncResponse(object):  # pylint: disable=too-few-public-methods
    """Proxy class to a future CliResponse, a response to an async comand.
       If any function of Cliresponse is called on an instance of this class
       the system will wait and block for the response to become ready.
    """

    def __init__(self, dut):
        try:
            self.logger = LogManager.get_bench_logger()
        except KeyError:
            self.logger = None
        self.response = None
        self.dut = dut

    def set_response(self, response):
        """
        Set the response, this function should not be called directly,
        the DUT will do it when a response is available.
        """
        if self.response is None:
            self.response = response

    def __wait_for_response(self):
        """
        Explicitelly wait and block for the response
        """
        if self.response is None:
            # The response will be filled - anyway - by the DUT,
            # there is no need to set it twice.
            self.dut._wait_for_exec_ready()  # pylint: disable=protected-access

    def __getattr__(self, name):
        """
        Forward calls and attribute lookup to the inner response once it is available
        """
        self.__wait_for_response()
        return getattr(self.response, name)

    def __str__(self):
        """
        Return the string representation of the response once it is available
        """
        self.__wait_for_response()
        return self.response.__str__()

    def __getitem__(self, item):
        """
        Index operator forwarded to the response once it is available
        """
        self.__wait_for_response()
        return self.response.__getitem__(item)



# pylint: disable=protected-access,missing-docstring,unused-argument,no-self-use
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

from icetea_lib.TestBench.RunnerSM import RunnerSM


class MockLogger(object):
    def __init__(self):
        pass

    def info(self, *args, **kwargs):
        pass

    def debug(self, *args, **kwargs):
        pass

    def error(self, *args, **kwargs):
        pass


def template_function(*args):
    pass


class MockArgs(object):  # pylint: disable=too-few-public-methods
    def __init__(self):
        self.pre_cmds = []
        self.post_cmds = []
        self.kill_putty = False


class RunnerSMTestcase(unittest.TestCase):

    def test_run(self):
        mocked_bench = mock.MagicMock()
        runner = RunnerSM(mocked_bench)
        runner.args = MockArgs()
        runner._load_plugins = template_function
        runner._init_duts = template_function
        runner._start_external_services = template_function
        runner._send_pre_commands = template_function
        runner._send_post_commands = template_function
        runner._duts_release = template_function
        runner._clear_sniffer = template_function
        runner._stop_external_services = template_function
        runner.kill_putty = template_function
        runner.logger = MockLogger()
        runner.run()
        runner.args.kill_putty = True
        runner.run()

    def test_benchteardown_errors(self):
        mocked_bench = mock.MagicMock()
        runner = RunnerSM(mocked_bench)
        mocked_bench.send_post_commands = mock.MagicMock(side_effect=[Exception, 0, 0, 0, 0])
        mocked_bench.duts_release = mock.MagicMock(side_effect=[0, 0, Exception, 0, 0])
        mocked_bench.clear_sniffer = mock.MagicMock(side_effect=[0, 0, 0, Exception, 0])
        mocked_bench.stop_external_services = mock.MagicMock(side_effect=[0, 0, 0, 0, Exception])
        runner._teardown_bench()
        mocked_bench.duts_release.assert_called_once()
        mocked_bench.clear_sniffer.assert_called_once()
        mocked_bench.stop_external_services.assert_called_once()
        runner._teardown_bench()
        self.assertEqual(mocked_bench.send_post_commands.call_count, 2)
        self.assertEqual(mocked_bench.duts_release.call_count, 2)
        self.assertEqual(mocked_bench.clear_sniffer.call_count, 2)
        self.assertEqual(mocked_bench.stop_external_services.call_count, 2)
        runner._teardown_bench()
        self.assertEqual(mocked_bench.send_post_commands.call_count, 3)
        self.assertEqual(mocked_bench.duts_release.call_count, 3)
        self.assertEqual(mocked_bench.clear_sniffer.call_count, 3)
        self.assertEqual(mocked_bench.stop_external_services.call_count, 3)
        runner._teardown_bench()
        self.assertEqual(mocked_bench.send_post_commands.call_count, 4)
        self.assertEqual(mocked_bench.duts_release.call_count, 4)
        self.assertEqual(mocked_bench.clear_sniffer.call_count, 4)
        self.assertEqual(mocked_bench.stop_external_services.call_count, 4)
        runner._teardown_bench()
        self.assertEqual(mocked_bench.send_post_commands.call_count, 5)
        self.assertEqual(mocked_bench.duts_release.call_count, 5)
        self.assertEqual(mocked_bench.clear_sniffer.call_count, 5)
        self.assertEqual(mocked_bench.stop_external_services.call_count, 5)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

ResultList module, contains the ResultList class, which is an object to store Result objects.
"""

from collections import Iterator

from icetea_lib.Reports.ReportJunit import ReportJunit
from icetea_lib.Reports.ReportHtml import ReportHtml
from icetea_lib.Reports.ReportConsole import ReportConsole
from icetea_lib.Result import Result


class ResultList(Iterator):
    """
    List of Result objects. Implements parts of Iterator interface to allow some ease of use.
    """
    def __init__(self):  # pylint: disable=super-init-not-called
        """
        Constructor for ResultList
        """
        self.data = []
        self.index = 0
        self.retries_left = 0

    def get(self, index=0):
        """
        Get object with index index.

        :param index: int
        :return: Result
        """
        return self.data[index]

    def append(self, result):
        """
        Append a new Result to the list.

        :param result: Result to append
        :return: Nothing
        :raises: TypeError if result is not Result or ResultList
        """
        if isinstance(result, Result):
            self.data.append(result)
        elif isinstance(result, ResultList):
            self.data += result.data
        else:
            raise TypeError('unknown result type')

        # @todo this could be used to generate html table after each test..
        # self._save_html_report({"NOTE": "TESTS EXECUTION IS ONGOING.."}, reload=5)

    def save(self, heads, console=True):
        """
        Create reports in different formats.

        :param heads: html table extra values in title rows
        :param console: Boolean, default is True. If set, also print out the console log.
        """
        # Junit
        self._save_junit()
        # HTML
        self._save_html_report(heads)
        if console:
            # Console print
            self._print_console_summary()

    def _save_junit(self):
        """
        Save Junit report.

        :return: Nothing
        """
        report = ReportJunit(self)
        file_name = report.get_latest_filename("result.junit.xml", "")
        report.generate(file_name)

        file_name = report.get_latest_filename("junit.xml", "../")
        report.generate(file_name)

    def _save_html_report(self, heads=None, refresh=None):
        """
        Save html report.

        :param heads: headers as dict
        :param refresh: Boolean, if True will add a reload-tag to the report
        :return: Nothing
        """
        report = ReportHtml(self)
        heads = heads if heads else {}
        test_report_filename = report.get_current_filename("html")
        report.generate(test_report_filename, title='Test Results', heads=heads, refresh=refresh)

        # Update latest.html in the log root directory
        latest_report_filename = report.get_latest_filename("html")
        report.generate(latest_report_filename, title='Test Results', heads=heads, refresh=refresh)

    def _print_console_summary(self):
        """
        Print the console report.

        :return: Nothing
        """
        ReportConsole(self).generate()

    def success_count(self):
        """
        Amount of passed test cases in this list.

        :return: integer
        """
        return len([i for i, result in enumerate(self.data) if result.success])

    def failure_count(self):
        """
        Amount of failed test cases in this list.

        :return: integer
        """
        return len([i for i, result in enumerate(self.data) if result.failure])

    def inconclusive_count(self):
        """
        Amount of inconclusive test cases in this list.

        :return: integer
        """
        inconc_count = len([i for i, result in enumerate(self.data) if result.inconclusive])
        unknown_count = len([i for i, result in enumerate(self.data) if result.get_verdict() ==
                             "unknown"])
        return inconc_count + unknown_count

    def retry_count(self):
        """
        Amount of retried test cases in this list.

        :return: integer
        """
        retries = len([i for i, result in enumerate(self.data) if result.retries_left > 0])
        return retries

    def skip_count(self):
        """
        Amount of skipped test cases in this list.

        :return: integer
        """
        return len([i for i, result in enumerate(self.data) if result.skip])

    def clean_fails(self):
        """
        Check if there are any fails that were not subsequently retried.

        :return: Boolean
        """
        for item in self.data:
            if item.failure and not item.retries_left > 0:
                return True
        return False

    def clean_inconcs(self):
        """
        Check if there are any inconclusives or uknowns that were not subsequently retried.

        :return: Boolean
        """
        for item in self.data:
            if (item.inconclusive or item.get_verdict() == "unknown") and not item.retries_left > 0:
                return True
        return False

    @property
    def skipped(self):
        """
        True if all results were skipped. Else False.

        :return: Boolean
        """
        return True if self.skip_count() == len(self) else False

    @property
    def inconclusive(self):
        """
        True if all failures were inconclusive. False if there are any fails or all tests passed.

        :return: Boolean
        """
        return True if self.inconclusive_count() and not self.failure else False

    @property
    def success(self):
        """
        If all tests passed, return True, else False.

        :return: Boolean
        """
        return True if not self.inconclusive and not self.failure else False

    @property
    def failure(self):
        """
        If any tests failed, return True, else False.

        :return: Boolean
        """
        for item in self.data:
            if item.failure:
                return True
        return False

    def get_verdict(self):
        """
        Get success state as a string from pass, fail, skip and inconclusive.

        :return: str
        """
        if self.success:
            return "pass"
        elif self.failure:
            return "fail"
        elif self.skipped:
            return "skip"
        return "inconclusive"

    def total_duration(self):
        """
        Sum of the durations of the tests in this list.

        :return: integer
        """
        durations = [result.duration for result in self.data]
        return sum(durations)

    def pass_rate(self, include_skips=False, include_inconclusive=False, include_retries=True):
        """
        Calculate pass rate for tests in this list.

        :param include_skips: Boolean, if True skipped tc:s will be included. Default is False
        :param include_inconclusive: Boolean, if True inconclusive tc:s will be included.
        Default is False.
        :param include_retries: Boolean, if True retried tc:s will be included in percentages.
        :return: Percentage in format .2f %
        """
        total = self.count()
        success = self.success_count()
        retries = self.retry_count()
        try:
            if include_inconclusive and include_skips and include_retries:
                val = 100.0*success/total
            elif include_inconclusive and include_skips and not include_retries:
                val = 100.0 * success / (total - retries)
            elif include_skips and include_retries and not include_inconclusive:
                inconcs = self.inconclusive_count()
                val = 100.0 * success / (total - inconcs)
            elif include_skips and not include_retries and not include_inconclusive:
                inconcs = self.inconclusive_count()
                val = 100.0 * success / (total - inconcs - retries)
            elif include_inconclusive and include_retries and not include_skips:
                skipped = self.skip_count()
                val = 100.0 * success / (total - skipped)
            elif include_inconclusive and not include_retries and not include_skips:
                skipped = self.skip_count()
                val = 100.0 * success / (total - skipped - retries)
            elif not include_inconclusive and not include_skips and include_retries:
                failures = self.failure_count()
                val = 100.0 * success / (failures + success)
            else:
                failures = self.clean_fails()
                val = 100.0 * success / (failures + success)
        except ZeroDivisionError:
            val = 0
        return format(val, '.2f') + " %"

    def get_summary(self):
        """
        Get a summary of this ResultLists contents as dictionary.

        :return: dictionary
        """
        return {
            "count": self.count(),
            "pass": self.success_count(),
            "fail": self.failure_count(),
            "skip": self.skip_count(),
            "inconclusive": self.inconclusive_count(),
            "retries": self.retry_count(),
            "duration": self.total_duration()
        }

    def count(self):
        """
        Return length of this list (amount of Results).

        :return: integer
        """
        return len(self)

    def __len__(self):
        """
        Return length of this list (amount of Results).

        :return: integer
        """
        return len(self.data)

    def __next__(self):
        """
        Get next Result from this list.

        :return: Result
        """
        return self.next()

    def next(self):
        """
        Implementation of next method from Iterator.

        :return: Result
        :raises: StopIteration if IndexError occurs.
        """
        try:
            result = self.data[self.index]
        except IndexError:
            self.index = 0
            raise StopIteration
        self.index += 1
        return result



# pylint: disable=missing-docstring

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import os
import jsonmerge
import mock

from requests.exceptions import RequestException
from requests import Response

from icetea_lib.tools.HTTP.Api import HttpApi

# Schema to make sure header fields are overwritten
SCHEMA = {
    "properties": {
        "mergeStrategy": "overwrite"
    }
}


class MockedRequestsResponse(object):  # pylint: disable=too-few-public-methods
    def __init__(self, status_code=200, json_data=None):
        self.json_data = json_data if json_data else {"key1": "value1"}
        self.status_code = status_code
        self.url = ''
        self.headers = {"head": "ers"}
        self.text = "This is test text"
        self.request = self

    def json(self):
        return self.json_data


class APITestCase(unittest.TestCase):

    def setUp(self):
        self.headers = {"accept-charset": "utf-8", "accept": "application/json"}
        self.host = "http://somesite.com"
        self.host2 = "http://somesite.com/api/"
        self.cert = "/path/to/cert.pem"
        self.http = None

    def test_init(self):
        self.http = HttpApi(self.host)
        self.assertEquals(self.http.host, self.host,
                          "HTTPApi not set up correctly, host names don't match")
        self. http = HttpApi(self.host, self.headers, self.cert)
        self.assertDictEqual(self.headers, self.http.defaultHeaders,
                             "HttpApi not set up correctly, headers not ok")
        self.assertEquals(self.cert, self.http.cert)

    # Test both new header field and overwrite
    def test_header_set(self):
        self.http = HttpApi(self.host, self.headers)
        # Test overwriting headers
        self.http.set_header("accept", "text/plain")
        self.assertEquals(self.http.defaultHeaders["accept"], "text/plain")
        # Test setting new headers
        self.http.set_header("accept-language", "en-US")
        self.assertEquals(self.http.defaultHeaders["accept-language"], "en-US")

    def test_header_merge(self):
        self.http = HttpApi(self.host, self.headers)
        headers = {"accept": "text/plain", "content-length": 348}
        ref = {"accept-charset": "utf-8", "accept": "text/plain", "content-length": 348}
        merger = jsonmerge.Merger(SCHEMA)
        heads = merger.merge(self.headers, headers)
        # Assert that new headers are the same as reference
        self.assertDictEqual(heads, ref, msg="Header merging does not work correctly")

    @mock.patch("icetea_lib.tools.HTTP.Api.requests.get")
    def test_url_combine(self, mock_get):
        self.http = HttpApi(self.host2)
        self.http.get("/test_path")
        mock_get.assert_called_with(self.host2 + "test_path", {}, headers={})
        mock_get.reset_mock()

        self.http.get("test_path")
        mock_get.assert_called_with(self.host2 + "test_path", {}, headers={})
        mock_get.reset_mock()

        self.http = HttpApi(self.host)
        self.http.get("test_path")
        mock_get.assert_called_with(self.host + "/test_path", {}, headers={})
        mock_get.reset_mock()

        self.http = HttpApi(self.host)
        self.http.get("/test_path")
        mock_get.assert_called_with(self.host + "/test_path", {}, headers={})

    @mock.patch("icetea_lib.tools.HTTP.Api.requests.get")
    def test_get(self, mock_requests_get):
        # First test successfull get request. Assert if get was called
        self.http = HttpApi(self.host)
        path = "/test"
        mock_requests_get.side_effect = iter([MockedRequestsResponse(),
                                              RequestException("Exception raised correctly"),
                                              MockedRequestsResponse(404),
                                              MockedRequestsResponse(200),
                                              MockedRequestsResponse(201)])
        resp = self.http.get(path)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_get.called, "Failed to call requests.get")
        path = "/exception"
        # Assert that RequestException is raised
        with self.assertRaises(RequestException, msg="request.get exception not raised properly"):
            self.http.get(path)

        mock_requests_get.reset_mock()
        path2 = "v2/"
        resp = self.http.get(path2)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_get.called, "Failed to call requests.get")

        mock_requests_get.reset_mock()
        path3 = "/v3/"
        self.http = HttpApi(self.host2)
        resp = self.http.get(path3)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_get.called, "Failed to call requests.get")

    @mock.patch("icetea_lib.tools.HTTP.Api.requests.post")
    def test_post(self, mock_requests_post):
        # Successfull post
        self.http = HttpApi(self.host)
        path = "/test"
        json = {"testkey1": "testvalue1"}
        mock_requests_post.side_effect = iter([MockedRequestsResponse(),
                                               RequestException("Exception raised correctly"),
                                               MockedRequestsResponse(404),
                                               MockedRequestsResponse(200),
                                               MockedRequestsResponse(201)])
        resp = self.http.post(path, json=json)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_post.called, "Failed to call requests.post")
        path = "/exception"
        # Assert that RequestException is raised
        with self.assertRaises(RequestException, msg="request.post exception not raised properly"):
            self.http.post(path, json=json)

        mock_requests_post.reset_mock()
        path2 = "v2/"
        resp = self.http.post(path2, json=json)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_post.called, "Failed to call requests.post")

        mock_requests_post.reset_mock()
        path3 = "/v3/"
        self.http = HttpApi(self.host2)
        resp = self.http.post(path3, json=json)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_post.called, "Failed to call requests.post")

    @mock.patch("icetea_lib.tools.HTTP.Api.requests.put")
    def test_put(self, mock_requests_put):
        # Successfull put
        self.http = HttpApi(self.host)
        path = "/test"
        data = {"testkey1": "testvalue1"}
        mock_requests_put.side_effect = iter([MockedRequestsResponse(),
                                              RequestException("Exception raised correctly"),
                                              MockedRequestsResponse(404),
                                              MockedRequestsResponse(200),
                                              MockedRequestsResponse(201)])
        resp = self.http.put(path, data=data)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_put.called, "Failed to call requests.put")
        path = "/exception"
        # Assert that RequestException is raised
        with self.assertRaises(RequestException, msg="request.put exception not raised properly"):
            self.http.put(path, data=data)

        mock_requests_put.reset_mock()
        path2 = "v2/"
        resp = self.http.put(path2, data=data)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_put.called, "Failed to call requests.put")

        mock_requests_put.reset_mock()
        path3 = "/v3/"
        self.http = HttpApi(self.host2)
        resp = self.http.put(path3, data=data)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_put.called, "Failed to call requests.put")

    @mock.patch("icetea_lib.tools.HTTP.Api.requests.patch")
    def test_patch(self, mock_requests_patch):
        # Successfull patch
        self.http = HttpApi(self.host)
        path = "/test"
        data = {"testkey1": "testvalue1"}
        mock_requests_patch.side_effect = iter([MockedRequestsResponse(),
                                                RequestException("Exception raised correctly"),
                                                MockedRequestsResponse(404),
                                                MockedRequestsResponse(200),
                                                MockedRequestsResponse(201)])
        resp = self.http.patch(path, data=data)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_patch.called, "Failed to call requests.patch")
        path = "/exception"
        # Assert that RequestException is raised
        with self.assertRaises(RequestException, msg="request.patch exception not raised properly"):
            self.http.patch(path, data=data)

        mock_requests_patch.reset_mock()
        path2 = "v2/"
        resp = self.http.patch(path2, data=data)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_patch.called, "Failed to call requests.patch")

        mock_requests_patch.reset_mock()
        path3 = "/v3/"
        self.http = HttpApi(self.host2)
        resp = self.http.patch(path3, data=data)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_patch.called, "Failed to call requests.patch")

    @mock.patch("icetea_lib.tools.HTTP.Api.requests.delete")
    def test_delete(self, mock_requests_delete):
        # Successfull delete
        self.http = HttpApi(self.host)
        path = "/test"
        mock_requests_delete.side_effect = iter([MockedRequestsResponse(),
                                                 RequestException("Exception raised correctly"),
                                                 MockedRequestsResponse(404),
                                                 MockedRequestsResponse(200),
                                                 MockedRequestsResponse(200)])
        resp = self.http.delete(path)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_delete.called, "Failed to call requests.delete")
        path = "/exception"
        # Assert that RequestException is raised
        with self.assertRaises(RequestException,
                               msg="request.delete exception not raised properly"):
            self.http.delete(path)

        mock_requests_delete.reset_mock()
        path2 = "v2/"
        resp = self.http.delete(path2)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_delete.called, "Failed to call requests.delete")

        mock_requests_delete.reset_mock()
        path3 = "/v3/"
        self.http = HttpApi(self.host2)
        resp = self.http.delete(path3)  # pylint: disable=unused-variable
        self.assertTrue(mock_requests_delete.called, "Failed to call requests.delete")

    @mock.patch("icetea_lib.tools.HTTP.Api.HttpApi.get")
    def test_huge_binary_content(self, mocked_get):
        var = os.urandom(10000000)
        for _ in range(6):
            var = var + os.urandom(10000000)
        response = Response()
        response._content = var  # pylint: disable=protected-access
        response.encoding = "utf-8"
        response.status_code = 200
        mocked_get.return_value = response
        self.http = HttpApi(self.host)
        resp = self.http.get("/")
        self.assertEqual(resp, response)


if __name__ == '__main__':
    unittest.main()



# pylint: disable=missing-docstring,pointless-string-statement

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import os
import sys
import unittest

import jsonmerge
import mock

import icetea_lib.tools.file.FileUtils as FileUtils
from icetea_lib.tools.file.SessionFiles import JsonFile


class FileTestCase(unittest.TestCase):

    def setUp(self):
        self.file_name = "test_file"
        self.file_path = os.path.join("path", "to", "right")
        self.file_path2 = os.path.join("path", "of", "no", "rights") + os.path.sep
        self.non_existing_file = "NoFile"
        self.test_json = {"test1": "value1", "test2": "value2", "test3": "value3", "test4": "test5"}
        self.test_json2 = {"Test": "Value", "Test1": "Value1", "Test2": "Value2"}
        self.jsonfile = JsonFile()

    '''
        FileUtils tests here
    '''

    @mock.patch("icetea_lib.tools.file.FileUtils.os.chdir")
    @mock.patch("icetea_lib.tools.file.FileUtils.os.remove")
    def test_remove_file(self, mock_rm, mock_chdir):
        mock_chdir.side_effect = iter([OSError, 1, 1, 1, 1])
        mock_rm.side_effect = iter([OSError, 1])
        path_nowhere = os.path.join("testPath", "to", "nowhere")
        path_somewhere = os.path.join("testPath", "to", "somewhere")
        with self.assertRaises(OSError):
            FileUtils.remove_file("testName", path_nowhere)
        with self.assertRaises(OSError):
            FileUtils.remove_file("testName", path_nowhere)
        self.assertTrue(FileUtils.remove_file("testName", path_somewhere))

    @mock.patch("icetea_lib.tools.file.FileUtils.os.chdir")
    @mock.patch("icetea_lib.tools.file.FileUtils.os.rename")
    def test_rename_file(self, mock_rn, mock_chdir):
        mock_chdir.side_effect = iter([OSError, 1, 1, 1, 1])
        mock_rn.side_effect = iter([OSError, 1])
        path_nowhere = os.path.join("testPath", "to", "nowhere")
        path_somewhere = os.path.join("testPath", "to", "somewhere")
        with self.assertRaises(OSError):
            FileUtils.rename_file("testName", "new_testName", path_nowhere)
        with self.assertRaises(OSError):
            FileUtils.rename_file("testName", "new_testName", path_nowhere)
        self.assertTrue(FileUtils.rename_file("testName", "new_testName", path_somewhere))

    '''
        JsonFile tests start here
    '''

    def test_writing_file(self):
        # run test for saving a json to a new file. File path should not exists so it is created.
        self.jsonfile.write_file(self.test_json, self.file_path, self.file_name, 2)
        self.assertTrue(os.path.exists(self.file_path + os.path.sep + self.file_name + ".json"))
        with open(self.file_path + os.path.sep + self.file_name + ".json", 'r') as file_opened:
            self.assertDictEqual(json.load(file_opened), self.test_json)

        # save a json to an old file to overwrite entire file
        self.jsonfile.write_file(self.test_json2, self.file_path, self.file_name, 2)
        with open(self.file_path + os.path.sep + self.file_name + ".json", 'r') as file_opened:
            self.assertDictEqual(json.load(file_opened), self.test_json2)

    @unittest.skipIf(sys.platform == 'win32', "os.chmod with 0o444 or stat.S_IREAD doesn't work")
    def test_writing_file_fail_windows(self):
        # run test to save json to a file with an invalid path
        os.makedirs(self.file_path2, 0o777)
        os.chmod(self.file_path2, 0o444)
        with self.assertRaises(IOError):
            self.jsonfile.write_file(self.test_json, self.file_path2, self.file_name, 2)

    def test_writing_with_filter(self):
        self.jsonfile.write_file(self.test_json, self.file_path, self.file_name, 2,
                                 keys_to_write=["test1", "test2"])
        ref = {"test1": self.test_json["test1"], "test2": self.test_json["test2"]}
        with open(self.file_path + os.path.sep + self.file_name + ".json", 'r') as file_opened:
            self.assertDictEqual(json.load(file_opened), ref)

        self.jsonfile.write_file(self.test_json, self.file_path, self.file_name, 2)
        self.jsonfile.write_values(self.test_json2, self.file_path, self.file_name, 2,
                                   keys_to_write=["Test"])
        with open(self.file_path + os.path.sep + self.file_name + ".json", 'r') as file_opened:
            self.assertDictEqual(jsonmerge.merge(self.test_json,
                                                 {"Test": self.test_json2["Test"]}),
                                 json.load(file_opened))


    @mock.patch("icetea_lib.tools.file.SessionFiles.JsonFile._write_json")
    @mock.patch("icetea_lib.tools.file.SessionFiles.os.path.exists")
    @mock.patch("icetea_lib.tools.file.SessionFiles.os.makedirs")
    def test_write_file_errors(self, mock_mkdir, mock_exists, mock_json):
        mock_mkdir.side_effect = iter([OSError])
        mock_exists.side_effect = iter([False, True, True])
        mock_json.side_effect = iter([EnvironmentError, ValueError])
        with self.assertRaises(OSError):
            self.jsonfile.write_file(self.test_json, self.file_path, self.file_name)
        with self.assertRaises(EnvironmentError):
            self.jsonfile.write_file(self.test_json, self.file_path, self.file_name)
        with self.assertRaises(ValueError):
            self.jsonfile.write_file(self.test_json, self.file_path, self.file_name)

    def test_reading_file(self):
        #Write test data
        self.jsonfile.write_file(self.test_json, self.file_path, self.file_name, 2)
        # read json from nonexisting file
        with self.assertRaises(IOError):
            self.jsonfile.read_file(self.file_path, self.non_existing_file)

        # read json from existing file
        data = self.jsonfile.read_file(self.file_path, self.file_name)
        self.assertDictEqual(data, self.test_json)
        #ToDo: read from malformed file

    def test_writing_values(self):
        #Test writing to a malformed file.
        self.jsonfile.write_file("", self.file_path, "malformed_" + self.file_name)
        with self.assertRaises(TypeError):
            self.jsonfile.write_values(self.test_json, self.file_path,
                                       "malformed_" + self.file_name)

        cwd = os.getcwd()
        os.chdir(self.file_path)
        open("empty_" + self.file_name + ".json", "w")
        os.chdir(cwd)
        self.assertEquals(self.jsonfile.write_values(self.test_json, self.file_path,
                                                     "empty_" + self.file_name),
                          os.path.join(self.file_path, "empty_" + self.file_name + ".json"))

        # save a json to a new file with write_values instead of write_file
        self.assertEqual(self.jsonfile.write_values(self.test_json, self.file_path,
                                                    self.file_name, 2),
                         self.file_path + os.path.sep + self.file_name + ".json")

        # save a json to an old file to append to existing data
        self.jsonfile.write_file(self.test_json, self.file_path, self.file_name, 2)
        self.jsonfile.write_values(self.test_json2, self.file_path, self.file_name, 2)
        with open(self.file_path + os.path.sep + self.file_name + ".json", 'r') as file_opened:
            self.assertDictEqual(jsonmerge.merge(self.test_json, self.test_json2),
                                 json.load(file_opened))

    @unittest.skipIf(sys.platform == 'win32', "os.chmod with 0o444 or stat.S_IREAD doesn't work")
    def test_writing_value_fail_windows(self):
        # Try adding content to a file with no write permissions
        os.makedirs(self.file_path2, 0o777)
        self.jsonfile.write_file(self.test_json2, self.file_path2, self.file_name, 2)
        os.chmod(self.file_path2, 0o444)
        with self.assertRaises(IOError):
            self.jsonfile.write_values(self.test_json, self.file_path2, self.file_name, 2)

    def test_reading_values(self):
        # Test writing to a malformed file.
        self.jsonfile.write_file("", self.file_path, "malformed_" + self.file_name)
        with self.assertRaises(KeyError):
            self.jsonfile.read_value("test1", self.file_path, "malformed_" + self.file_name)

        # read json from nonexisting file
        with self.assertRaises(IOError):
            self.jsonfile.read_value("Test1", self.file_path, self.file_name)

        # Create test data for reading
        self.jsonfile.write_file(self.test_json, self.file_path, self.file_name, 2)
        # read json from existing file

        self.assertEqual("value1", self.jsonfile.read_value("test1",
                                                            self.file_path, self.file_name))

        # try to read nonexisting key
        with self.assertRaises(KeyError):
            self.jsonfile.read_value("value1", self.file_path, self.file_name)

    def tearDown(self):
        # Do tearDown stuff.
        if os.path.exists(os.path.join(self.file_path, "malformed_" + self.file_name + ".json")):
            FileUtils.remove_file("malformed_" + self.file_name + ".json",
                                  self.file_path + os.path.sep)

        if os.path.exists(os.path.join(self.file_path, "empty_" + self.file_name + ".json")):
            FileUtils.remove_file("empty_" + self.file_name + ".json", self.file_path + os.path.sep)

        if os.path.exists(self.file_path + os.path.sep + self.file_name + ".json"):
            FileUtils.remove_file(self.file_name + ".json", self.file_path + os.path.sep)
            os.removedirs(self.file_path + os.path.sep)
        if os.path.exists(self.file_path2):
            os.chmod(self.file_path2, 0o777)
            if os.path.exists(self.file_path2 + self.file_name + ".json"):
                FileUtils.remove_file(self.file_name + ".json", self.file_path2)
            os.removedirs(self.file_path2)


if __name__ == '__main__':
    unittest.main()



# pylint: disable=no-member,unused-import
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

BenchFunctions module. Contains functions for the test bench.
"""

import os
import subprocess
import sys
import time

import icetea_lib.LogManager as LogManager
from icetea_lib.Searcher import verify_message


class BenchFunctions(object):
    """
    A collection of functions that add different functionalities to the test bench.
    """
    def __init__(self, resource_configuration, resources, configurations, **kwargs):
        super(BenchFunctions, self).__init__(**kwargs)
        self._resource_configuration = resource_configuration
        self._resources = resources
        self._configurations = configurations
        self._logger = LogManager.get_dummy_logger()

    def init(self, logger=None):
        """
        Set logger.
        """
        if logger:
            self._logger = logger

    # input data from user
    def input_from_user(self, title=None):  # pylint: disable=no-self-use
        """
        Input data from user.

        :param title: Title as string
        :return: stripped data from stdin.
        """
        if title:
            print(title)
        print("Press [ENTER] to continue")
        resp = sys.stdin.readline().strip()
        if resp != '':
            return resp.strip()
        return ""

    def open_node_terminal(self, k='*', wait=True):
        """
        Open Putty (/or kitty if exists)

        :param k: number 1.<max duts> or '*' to open putty to all devices
        :param wait: wait while putty is closed before continue testing
        :return: Nothing
        """
        if k == '*':
            for ind in self._resource_configuration.get_dut_range():
                self.open_node_terminal(ind, wait)
            return

        if not self._resources.is_my_dut_index(k):
            return

        params = '-serial ' + self._resources.duts[k - 1].comport + ' -sercfg ' + str(
            self._resources.duts[k - 1].serialBaudrate)

        putty_exe = self._configurations.env['extApps']['puttyExe']
        if os.path.exists(self._configurations.env['extApps']['kittyExe']):
            putty_exe = self._configurations.env['extApps']['kittyExe']

        if "kitty.exe" in putty_exe:
            params = params + ' -title "' + self._resources.duts[k - 1].comport

            params += ' - ' + self._configurations.test_name
            params += ' | DUT' + str(k) + ' ' + self._resources.get_dut_nick(k) + '"'
            params += ' -log "' + LogManager.get_testcase_logfilename('DUT%d.manual' % k) + '"'

        if os.path.exists(putty_exe):

            command = putty_exe + ' ' + params
            self._logger.info(command)
            if wait:
                if self._resources.is_my_dut_index(k):
                    self._resources.duts[k - 1].close_dut()
                    self._resources.duts[k - 1].close_connection()
                    self._resources.resource_provider.allocator.release(
                        dut=self._resources.duts[k - 1])
                    process = subprocess.Popen(command)
                    time.sleep(2)
                    process.wait()
                    self._resources.duts[k - 1].open_dut()
            else:
                subprocess.Popen(command, close_fds=True)
        else:
            self._logger.warning('putty not exists in path: %s', putty_exe)

    def delay(self, seconds):
        """
        Sleep command.

        :param seconds: Amount of seconds to sleep.
        :return: Nothing
        """
        self._logger.debug("Waiting for %i seconds", seconds)
        if seconds < 30:
            time.sleep(seconds)
        else:
            while seconds > 10:
                self._logger.debug("Still waiting... %i seconds remain", seconds)
                time.sleep(10)
                seconds = seconds - 10
            time.sleep(seconds)

    def verify_trace_skip_fail(self, k, expected_traces):
        """
        Shortcut to set break_in_fail to False in verify_trace.

        :param k: nick or index of dut.
        :param expected_traces: Expected traces as a list or string
        :return: boolean
        """
        return self.verify_trace(k, expected_traces, False)

    def verify_trace(self, k, expected_traces, break_in_fail=True):
        """
        Verify that traces expected_traces are found in dut traces.

        :param k: index or nick of dut whose traces are to be used.
        :param expected_traces: list of expected traces or string
        :param break_in_fail: Boolean, if True raise LookupError if search fails
        :return: boolean.
        :raises: LookupError if search fails.
        """
        if isinstance(k, str):
            dut_index = self._resources.get_dut_index(k)
            return self.verify_trace(dut_index, expected_traces, break_in_fail)

        # If expectedTraces given as a String (expecting only a certain trace), wrap it in a list.
        if isinstance(expected_traces, str):
            expected_traces = [expected_traces]

        status = True
        try:
            status = verify_message(self._resources.duts[k - 1].traces, expected_traces)
        except TypeError as inst:
            status = False
            if break_in_fail:
                raise inst
        if status is False and break_in_fail:
            raise LookupError("{} not found in traces.".format(expected_traces))
        return status

    def get_time(self):  # pylint: disable=no-self-use
        """
        Get timestamp using time.time().

        :return: timestamp
        """
        return time.time()



"""
Copyright 2019 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

DutMbed module contains the DutMbed Dut subclass.
"""
# pylint: disable=too-many-branches,too-many-arguments
from prettytable import PrettyTable

from icetea_lib.LogManager import get_external_logger
from icetea_lib.Plugin.plugins.LocalAllocator.DutSerial import DutSerial
from icetea_lib.DeviceConnectors.Dut import DutConnectionError
from icetea_lib.build import Build

try:
    from mbed_flasher.flash import Flash
except ImportError:
    Flash = None

try:
    from mbed_flasher.common import FlashError

    FLASHER_ERRORS = (FlashError, SyntaxError, NotImplementedError)
except ImportError:
    FlashError = None
    FLASHER_ERRORS = (SyntaxError, NotImplementedError)


class DutMbed(DutSerial):
    """
    DutMbed, child of DutSerial. Mbed device over serial connection.
    """
    def __init__(self, name='mbed', port=None, baudrate=115200, config=None,
                 ch_mode_config=None, serial_config=None, params=None):
        """
        Mbed device over serial connection.

        :param name: Dut name
        :param port: Serial port name
        :param baudrate: Baudrate, int
        :param config: configuration dict
        :param ch_mode_config: dict
        :param serial_config: dict
        :param params: dict
        """
        super(DutMbed, self).__init__(name, port, baudrate, config, ch_mode_config,
                                      serial_config, params)

    def flash(self, binary_location=None, forceflash=None):
        """
        Flash a binary to the target device using mbed-flasher.

        :param binary_location: Binary to flash to device.
        :param forceflash: Not used.
        :return: False if an unknown error was encountered during flashing.
        True if flasher retcode == 0
        :raises: ImportError if mbed-flasher not installed.
        :raises: DutConnectionError if flashing fails.
        """
        if not Flash:
            self.logger.error("Mbed-flasher not installed!")
            raise ImportError("Mbed-flasher not installed!")

        try:
            # create build object
            self.build = Build.init(binary_location)
        except NotImplementedError as error:
            self.logger.error("Build initialization failed. "
                              "Check your build location.")
            self.logger.debug(error)
            raise DutConnectionError(error)

        # check if need to flash - depend on forceflash -option
        if not self._flash_needed(forceflash=forceflash):
            self.logger.info("Skipping flash, not needed.")
            return True
        # initialize mbed-flasher with proper logger
        logger = get_external_logger("mbed-flasher", "FLS")
        flasher = Flash(logger=logger)

        if not self.device:
            self.logger.error("Trying to flash device but device is not there?")
            return False

        try:
            buildfile = self.build.get_file()
            if not buildfile:
                raise DutConnectionError("Binary {} not found".format(buildfile))
            self.logger.info('Flashing dev: %s', self.device['target_id'])
            target_id = self.device.get("target_id")
            retcode = flasher.flash(build=buildfile, target_id=target_id,
                                    device_mapping_table=[self.device])
        except FLASHER_ERRORS as error:
            if error.__class__ == NotImplementedError:
                self.logger.error("Flashing not supported for this platform!")
            elif error.__class__ == SyntaxError:
                self.logger.error("target_id required by mbed-flasher!")
            if FlashError is not None:
                if error.__class__ == FlashError:
                    self.logger.error("Flasher raised the following error: %s Error code: %i",
                                      error.message, error.return_code)
            raise DutConnectionError(error)
        if retcode == 0:
            self.dutinformation.build_binary_sha1 = self.build.sha1
            return True
        self.dutinformation.build_binary_sha1 = None
        return False

    def _flash_needed(self, **kwargs):
        """
        Check if flashing is needed. Flashing can be skipped if resource binary_sha1 attribute
        matches build sha1 and forceflash is not True.

        :param kwargs: Keyword arguments (forceflash: Boolean)
        :return: Boolean
        """
        forceflash = kwargs.get("forceflash", False)
        cur_binary_sha1 = self.dutinformation.build_binary_sha1
        if not forceflash and self.build.sha1 == cur_binary_sha1:
            return False
        return True

    def print_info(self):
        """
        Prints Dut information nicely formatted into a table.

        :return: Nothing
        """
        table = PrettyTable()
        start_string = "DutMbed {} \n".format(self.name)
        row = []
        info_string = ""
        if self.config:
            info_string = info_string + "Configuration for this DUT:\n\n {} \n".format(self.config)
        if self.comport:
            table.add_column("COM port", [])
            row.append(self.comport)
        if self.port:
            if hasattr(self.port, "baudrate"):
                table.add_column("Baudrate", [])
                row.append(self.port.baudrate)
            if hasattr(self.port, "xonxoff"):
                table.add_column("XON/XOFF", [])
                row.append(self.port.xonxoff)
            if hasattr(self.port, "timeout"):
                table.add_column("Timeout", [])
                row.append(self.port.timeout)
            if hasattr(self.port, "rtscts"):
                table.add_column("RTSCTS", [])
                row.append(self.port.rtscts)
        if self.location:
            table.add_column("Location", [])
            row.append("X = {}, Y = {}".format(self.location.x_coord, self.location.y_coord))
        self.logger.info(start_string)
        self.logger.debug(info_string)
        table.add_row(row)
        print(table)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

NodeEndPoint class. Just a wrapper for a single Dut.
"""


class NodeEndPoint(object):  # pylint: disable=too-few-public-methods
    """
    Wrapper for a dut, contains a shortcut to send commands to this dut specifically.
    """

    def __init__(self, bench, endpoint_id):
        self.bench = bench
        self.endpoint_id = endpoint_id

    def command(self, cmd, expected_retcode=0):  # pylint: disable=invalid-name
        # expected_retcode kwd argument is used in many test cases, we cannot rename it.
        """
        Shortcut for sending a command to this node specifically.
        :param cmd: Command to send
        :param expected_retcode: Expected return code as int, default is 0
        :return: CliResponse
        """
        return self.bench.execute_command(self.endpoint_id, cmd, expected_retcode=expected_retcode)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# pylint: disable=ungrouped-imports,unused-argument

import importlib
import inspect
import logging
import os
import sys
from sys import version_info
from sys import platform as _platform
import platform
import re
import string

from pkg_resources import require, DistributionNotFound
from six import iteritems

try:
    from queue import Empty
except ImportError:
    from Queue import Empty


UNIXPLATFORM = _platform == "linux" or _platform == "linux2" or _platform == "darwin"
IS_PYTHON3 = version_info[0] == 3


def get_fw_name():
    """
    :return: Framework name as str
    """
    return "Icetea"


def get_fw_version():
    """
    Try to get version of the framework. First try pkg_resources.require, if that fails
    read from setup.py
    :return: Version as str
    """
    version = 'unknown'
    try:
        pkg = require(get_fw_name())[0]
    except DistributionNotFound:
        # Icetea is not installed. try to read version string from setup.py
        try:
            setup_path = os.path.abspath(os.path.dirname(__file__)+'/../..')
            with open(os.path.join(setup_path, 'setup.py')) as setup_file:
                lines = setup_file.readlines()
                for line in lines:
                    match = re.search(r"VERSION = \"([\S]{5,})\"", line)
                    if match:
                        version = match.group(1)
                        break
        except Exception:  # pylint: disable=broad-except
            pass
    else:
        version = "-rc".join(pkg.version.split("rc"))
    return version


# calculate sha1 for a file
def sha1_of_file(filepath):
    """
    Get sha1 of file
    :param filepath: File to hash
    :return: sha1 hash of file or None
    """
    import hashlib
    try:
        with open(filepath, 'rb') as file_to_hash:
            return hashlib.sha1(file_to_hash.read()).hexdigest()
    except:  # pylint: disable=bare-except
        return None


# Check if number is integer or not
def check_int(integer):
    """
    Check if number is integer or not.

    :param integer: Number as str
    :return: Boolean
    """
    if not isinstance(integer, str):
        return False
    if integer[0] in ('-', '+'):
        return integer[1:].isdigit()
    return integer.isdigit()


# Convert string to the number
def num(integer):
    """
    Convert string to the number.

    :param integer: String to convert
    :return: int or None
    """
    try:
        if check_int(integer):
            return int(integer)
        return -1
    except ValueError:
        return -1


# Check if PID (process id) is running
def is_pid_running(pid):
    """
    Check if PID (process id) is running.

    :param pid: Pid to check
    :return : Boolean
    """
    if platform.system() == "Windows":
        return _is_pid_running_on_windows(pid)
    return _is_pid_running_on_unix(pid)


def _is_pid_running_on_unix(pid):
    """
    Check if PID is running for Unix systems.
    """
    try:
        os.kill(pid, 0)
    except OSError as err:
        # if error is ESRCH, it means the process doesn't exist
        return not err.errno == os.errno.ESRCH
    return True


def _is_pid_running_on_windows(pid):
    """
    Check if PID is running for Windows systems
    """
    import ctypes.wintypes

    kernel32 = ctypes.windll.kernel32
    handle = kernel32.OpenProcess(1, 0, pid)
    if handle == 0:
        return False
    exit_code = ctypes.wintypes.DWORD()
    ret = kernel32.GetExitCodeProcess(handle, ctypes.byref(exit_code))
    is_alive = (ret == 0 or exit_code.value == _STILL_ALIVE)  # pylint: disable=undefined-variable
    kernel32.CloseHandle(handle)
    return is_alive

ansi_pattern = r'\033\[((?:\d|;)*)([a-zA-Z])'  # pylint: disable=invalid-name
ansi_eng = re.compile(ansi_pattern)  # pylint: disable=invalid-name


def strip_escape(string='', encoding="utf-8"):  # pylint: disable=redefined-outer-name
    """
    Strip escape characters from string.

    :param string: string to work on
    :param encoding: string name of the encoding used.
    :return: stripped string
    """
    matches = []
    try:
        if hasattr(string, "decode"):
            string = string.decode(encoding)
    except Exception: # pylint: disable=broad-except
        # Tried to decode something that is not decodeable in the specified encoding. Let's just
        # move on.
        pass
    try:
        for match in ansi_eng.finditer(string):
            matches.append(match)
    except TypeError as error:
        raise TypeError("Unable to strip escape characters from data {}: {}".format(
            string, error))
    matches.reverse()
    for match in matches:
        start = match.start()
        end = match.end()
        string = string[0:start] + string[end:]
    return string


def load_class(full_class_string, verbose=False, silent=False):
    """
    dynamically load a class from a string
    """
    if not isinstance(full_class_string, str):
        if not silent:
            print("Error, loadClass: input not a string: %s" % str(full_class_string))
        return None
    if len(full_class_string) == 0:  # pylint: disable=len-as-condition
        if not silent:
            print("Error, loadClass: empty string")
        return None
    class_data = full_class_string.split(".")
    module_path = ".".join(class_data[:-1])
    class_str = class_data[-1]
    try:
        module_ = importlib.import_module(module_path)
        # Finally, we retrieve the Class
        return getattr(module_, class_str)
    except AttributeError:
        return None
    except Exception:
        raise


def import_module(modulename):
    """
    Static method for importing module modulename. Can handle relative imports as well.

    :param modulename: Name of module to import. Can be relative
    :return: imported module instance.
    """
    module = None
    try:
        module = importlib.import_module(modulename)
    except ImportError:
        # If importing fails we see if the modulename has dots in it, split the name.
        if "." in modulename:
            modules = modulename.split(".")
            package = ".".join(modules[1:len(modules)])
            # Might raise an ImportError again. If so, we really failed to import the module.
            module = importlib.import_module(package)
        else:
            # No dots, really unable to import the module. Raise.
            raise
    return module


def flush_queue(queue):
    """
    Flush the queue.

    :param queue: queue to flush
    :return: Nothing
    """
    while True:
        try:
            queue.get(block=False)
        except Empty:
            break


def get_abs_path(relative_path):
    """
    Get absolute path for relative path.

    :param relative_path: Relative path
    :return: absolute path
    """
    abs_path = os.path.sep.join(
        os.path.abspath(sys.modules[__name__].__file__).split(os.path.sep)[:-1])
    abs_path = os.path.abspath(abs_path + os.path.sep + relative_path)
    return abs_path


def get_pkg_version(pkg_name, parse=False):
    """
    Verify and get installed python package version.

    :param pkg_name:    python package name
    :param parse: parse version number with pkg_resourc.parse_version -function
    :return: None if pkg is not installed,
    otherwise version as a string or parsed version when parse=True
    """
    import pkg_resources  # part of setuptools
    try:
        version = pkg_resources.require(pkg_name)[0].version
        return pkg_resources.parse_version(version) if parse else version
    except pkg_resources.DistributionNotFound:
        return None


def get_number(string_nums):
    """
    :param string_nums: string, where to look up numbers(integer)
    :return: number or None if number(integer) is not found
    """
    match = re.match(r".*([\d]{1,})", string_nums)
    if match:
        return int(match.group(1))
    return None


ogcounter = 0  # pylint: disable=invalid-name


def generate_object_graphs_by_class(classlist):
    """
    Generate reference and backreference graphs
    for objects of type class for each class given in classlist.
    Useful for debugging reference leaks in framework etc.

    Usage example to generate graphs for class "someclass":
    >>> import someclass
    >>> someclassobject = someclass()
    >>> generate_object_graphs_by_class(someclass)

    Needs "objgraph" module installed.
    """
    try:
        import objgraph
        import gc
    except ImportError:
        return
    graphcount = 0
    if not isinstance(classlist, list):
        classlist = [classlist]
    for class_item in classlist:
        for obj in gc.get_objects():
            if isinstance(obj, class_item):
                graphcount += 1
                objgraph.show_refs([obj], filename='%d_%s_%d_refs.png' % (
                    ogcounter, obj.__class__.__name__, graphcount))
                objgraph.show_backrefs([obj], filename='%d_%s_%d_backrefs.png' % (
                    ogcounter, obj.__class__.__name__, graphcount))


def combine_urls(path1, path2):
    """
    Returns the combination of two urls and checks that there are no double slashes at the seam.
    TODO: Extend to check for other anomalies as well.

    :param path1: First part of the url
    :param path2: Second part of the url
    :return: Combination of the paths with double slashes removed
    """
    if path1.endswith("/") and path2.startswith("/"):
        path2 = path2.replace("/", "", 1)
    elif path1.endswith("/") is False and path2.startswith("/") is False:
        path2 = "/" + path2
    return path1 + path2


def recursive_dictionary_get(keys, dictionary):
    """
    Gets contents of requirement key recursively so users can search
    for specific keys inside nested requirement dicts.

    :param keys: key or dot separated string of keys to look for.
    :param dictionary: Dictionary to search from
    :return: results of search or None
    """
    if "." in keys and len(keys) > 1:
        key = keys.split(".", 1)
        new_dict = dictionary.get(key[0])
        # Make sure that the next level actually has a dict we can continue the search from.
        if not new_dict or not hasattr(new_dict, "get"):
            return None
        return recursive_dictionary_get(key[1], new_dict)
    else:
        return dictionary.get(keys) if (dictionary and hasattr(dictionary, "get")) else None


def test_case(testcasebase, **kwargs):
    """
    Decorator which allow definition of test cases as simple functions.

    :param testcasebase: this is the base class that will be used to create
    the test case. It is exected that this base class implement __init__,
    setUp and tearDown
    :param kwargs: Dictionary of arguments that will be passed as initialization
    parameters of the test case
    """
    def wrap(case_function):  # pylint: disable=missing-docstring
        kwargs['name'] = name = kwargs.get('name', case_function.__name__)
        class_name = "class_" + name
        func_globals = case_function.__globals__ if IS_PYTHON3 else case_function.func_globals
        func_globals[class_name] = type(
            class_name,
            (testcasebase, object), {
                '__init__': lambda self: testcasebase.__init__(self, **kwargs),
                'IS_TEST': True,
                'case': case_function
            }
        )
        return case_function
    return wrap


def remove_empty_from_dict(dictionary):
    """
    Remove empty items from dictionary d

    :param dictionary:
    :return:
    """
    if isinstance(dictionary, dict):
        return dict(
            (k,
             remove_empty_from_dict(v)) for k, v in iteritems(
                 dictionary) if v and remove_empty_from_dict(v))
    elif isinstance(dictionary, list):
        return [remove_empty_from_dict(v) for v in dictionary if v and remove_empty_from_dict(v)]
    return dictionary


def hex_escape_str(original_str):
    """
    Function to make sure we can generate proper string reports.
    If character is not printable, call repr for that character.
    Finally join the result.
    :param original_str: Original fail reason as string.
    :return: string
    """
    new = []
    for char in original_str:
        if str(char) in string.printable:
            new.append(str(char))
            continue

        if IS_PYTHON3:
            new.append(str(char).encode("unicode_escape").decode("ascii"))
        else:
            new.append(repr(char).replace("'", ""))
    return "".join(new)


def set_or_delete(dictionary, key, value):
    """
    Set value as value of dict key key. If value is None, delete key key from dict.

    :param dictionary: Dictionary to work on.
    :param key: Key to set or delete. If deleting and key does not exist in dict, nothing is done.
    :param value: Value to set. If value is None, delete key.
    :return: Nothing, modifies dict in place.
    """
    if value:
        dictionary[key] = value
    else:
        if dictionary.get(key):
            del dictionary[key]


def split_by_n(seq, n_var):
    """
    A generator to divide a sequence into chunks of n units.
    """
    while seq:
        yield seq[:n_var]
        seq = seq[n_var:]


def getargspec(fnct):
    """
    Compatibility wrapper for inspect.getargspec, which is deprecated in python 3.

    :param fnct: Function to inspect
    :return: Named tuple
    """
    if IS_PYTHON3:
        return inspect.getfullargspec(fnct)  # pylint: disable=no-member
    return inspect.getargspec(fnct)


def is_test(method_name):
    """
    :param method_name: Method name to check.
    :return: True if method name starts with test or if it is case.
    """
    # TODO: Implement other methods of recognicing which are tests.
    return method_name == "case"  # or method_name.startswith("test_")


def test_methods(target, test_filter=is_test):
    """
    Return a list of test methods.

    :param target: Target object whose methods to find.
    :param test_filter: filtering function.
    :return: list
    """
    def is_fnc(method_name):
        """
        Check that method name attr of target object is callable.
        :param method_name: Attribute name as string
        :return: True if target.method_name is callable
        """
        return callable(getattr(target, method_name))
    return [method_name for method_name in dir(target) if
            test_filter(method_name) and is_fnc(method_name)]


def initLogger(name):  # pylint: disable=invalid-name
    '''
    Initializes a basic logger. Can be replaced when constructing the
    HttpApi object or afterwards with setter
    '''
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    # Skip attaching StreamHandler if one is already attached to logger
    if not getattr(logger, "streamhandler_set", None):
        consolehandler = logging.StreamHandler()
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        consolehandler.setFormatter(formatter)
        consolehandler.setLevel(logging.INFO)
        logger.addHandler(consolehandler)
        logger.streamhandler_set = True
    return logger

def _create_combined_set(words, startindex):
    """
    Combines a single string wrapped in parenthesis from a list of words.
    Example: ['feature1', 'or', '(feature2', 'and', 'feature3)'] -> '(feature2 and feature3)'
    :param words: List of words
    :param startindex: Index of starting parenthesis.
    :return: string
    """
    counter = 0
    for i, word in enumerate(words[startindex:]):
        if ")" in word and counter == 0:
            return " ".join(words[startindex:startindex + i + 1]), i
        elif ")" in word and counter != 0:
            amount = word.count(")")
            counter -= amount
        if "(" in word:
            counter += word.count("(")
        if counter == 0:
            return " ".join(words[startindex:startindex + i + 1]), i
    return None, 0


def _create_combined_words(words, startindex):
    """
    Helper for create_match_bool, used to combine words inside single quotes from a list into a
    single string.
    :param words: List of words.
    :param startindex: Index where search is started.
    :return: (str, int) or (None, 0) if no closing quote is found.
    """
    for i, word in enumerate(words[startindex+1:]):
        if "'" in word:
            return " ".join(words[startindex:startindex+i+2]), i+1
    return None, 0


# pylint: disable=too-many-branches
def create_match_bool(words_as_string, eval_function, eval_function_args):
    """
    Evaluates components of words_as_string step by step into a single boolean value using
    eval_function to evaluate each separate component of words_as_string into booleans.
    :param words_as_string: String to evaluate
    :param eval_function: Function for evaluating each component of the string
    :param eval_function_args: Arguments for eval_function as list or tuple
    :return: Boolean
    :raises: SyntaxError if eval raises one.
    """
    new_set = []
    count = 0
    words_as_string = words_as_string.replace(",", " or ")
    words = words_as_string.split(" ")
    # First we combine items that contain spaces back into single entries
    for i, word in enumerate(words):
        if count != 0:
            count -= 1
            continue
        if "'" in word:
            # Combine stuff between quotes into single entry
            combined_data, count = _create_combined_words(words, i)
            if combined_data is None:
                raise SyntaxError("Missing closing quote for: {}".format(word))
            # Count needed to skip over the N next items while we continue
            new_set.append(combined_data)
        else:
            new_set.append(word)

    newest_set = []

    # Then we combine elements inside parentheses into single entries.
    for i, word in enumerate(new_set):
        if count != 0:
            count -= 1
            continue
        if "(" in word:
            # Combine stuff between parentheses into single entry that can be reduced later.
            combined_data, count = _create_combined_set(new_set, i)
            if combined_data is None:
                raise SyntaxError("Missing closing parenthesis for: {}".format(word))
            # Count needed to skip over the N next items while we continue
            newest_set.append(combined_data)
        else:
            newest_set.append(word)

    # Finally we enter deeper inside the parenthesis structure and start working upwards
    for i, word in enumerate(newest_set):
        if "(" in word:
            # Still parentheses remaining, we must go deeper.
            result = create_match_bool(word[1:len(word) - 1], eval_function,
                                       eval_function_args)
            newest_set[i] = str(result)
    for i, word in enumerate(newest_set):
        if word not in ["True", "False", "or", "and", "not"]:
            newest_set[i] = str(eval_function(word.replace("'", ""), eval_function_args))
    result = eval(" ".join(newest_set))  # pylint: disable=eval-used
    return result


def find_duplicate_keys(data):
    """
    Find duplicate keys in a layer of ordered pairs. Intended as the object_pairs_hook callable
    for json.load or loads.

    :param data: ordered pairs
    :return: Dictionary with no duplicate keys
    :raises ValueError if duplicate keys are found
    """
    out_dict = {}
    for key, value in data:
        if key in out_dict:
            raise ValueError("Duplicate key: {}".format(key))
        out_dict[key] = value
    return out_dict


class Singleton(type):
    """
    Singleton metaclass implementation:
    http://stackoverflow.com/questions/6760685/creating-a-singleton-in-python
    """

    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]



# pylint: disable=no-member,too-many-instance-attributes
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Mixer containing helpers for handling Results and ResultLists.
"""

import os
from icetea_lib.ReturnCodes import ReturnCodes
import icetea_lib.LogManager as LogManager
from icetea_lib.Result import Result
from icetea_lib.ResultList import ResultList
from icetea_lib.tools.GitTool import get_git_info


# @todo this needs some more work
class Results(object):
    """
    ResultMixer manage test results and verdicts.
    It provide public API get_result() for TestManagement.
    """
    def __init__(self, logger, resources, configuration, args, **kwargs):
        super(Results, self).__init__(**kwargs)
        self._result_list = ResultList()
        self._retcode = ReturnCodes.RETCODE_SUCCESS
        self._failreason = ''
        self._logger = logger
        self._configuration = configuration
        self._args = args
        self._resources = resources

    def init(self, logger=None):
        """
        Initialize the internal ResultList.
        :return: Nothing
        """
        if logger:
            self._logger = logger
        self._result_list = ResultList()

    @staticmethod
    def create_new_result(verdict, retcode, duration, input_data):
        """
        Create a new Result object with data in function arguments.

        :param verdict: Verdict as string
        :param retcode: Return code as int
        :param duration: Duration as time
        :param input_data: Input data as dictionary
        :return: Result
        """
        new_result = Result(input_data)
        new_result.set_verdict(verdict, retcode, duration)
        return new_result

    def add_new_result(self, verdict, retcode, duration, input_data):
        """
        Add a new Result to result object to the internal ResultList.

        :param verdict: Verdict as string
        :param retcode: Return code as int
        :param duration: Duration as time
        :param input_data: Input data as dict
        :return: Result
        """
        new_result = Results.create_new_result(verdict, retcode, duration, input_data)
        self._result_list.append(new_result)
        return new_result

    @property
    def retcode(self):
        """
        Getter for return code.

        :return: int
        """
        return self._retcode

    @retcode.setter
    def retcode(self, value):
        """
        Setter for retcode.

        :param value: int
        :return: Nothing
        """
        self._retcode = value

    def set_failure(self, retcode, reason):
        """
        Set internal state to reflect failure of test.

        :param retcode: return code
        :param reason: failure reason as string
        :return: Nothing
        """
        if self._resources.resource_provider:
            if hasattr(self._resources.resource_provider, "allocator"):
                # Check for backwards compatibility with older pyclient versions.
                if hasattr(self._resources.resource_provider.allocator, "get_status"):
                    pr_reason = self._resources.resource_provider.allocator.get_status()
                    if pr_reason:
                        reason = "{}. Other error: {}".format(pr_reason, reason)
                        retcode = ReturnCodes.RETCODE_FAIL_DUT_CONNECTION_FAIL

        if self.retcode is None or self.retcode == ReturnCodes.RETCODE_SUCCESS:
            self.retcode = retcode
            self._failreason = reason
            self._logger.error("Test Case fails because of: %s", reason)
        else:
            self._logger.info("another fail reasons: %s", reason)

    def get_results(self):
        """
        Getter for internal _results variable.
        """
        return self._result_list

    def get_result(self, tc_file=None):
        """
        Generate a Result object from this test case.

        :param tc_file: Location of test case file
        :return: Result
        """
        self.append_result(tc_file)
        return self._result_list.data[0]

    def set_results(self, value):
        """
        Setter for _result_list.

        :param value: ResultList
        :return: Nothing
        """
        self._result_list = value

    def append_result(self, tc_file=None):
        """
        Append a new fully constructed Result to the internal ResultList.

        :param tc_file: Test case file path
        :return: Nothing
        """
        result = Result()

        result.set_tc_metadata(self._configuration.config)
        tc_rev = get_git_info(self._configuration.get_tc_abspath(tc_file),
                              verbose=self._args.verbose)
        if self._logger:
            self._logger.debug(tc_rev)
        result.set_tc_git_info(tc_rev)
        result.component = self._configuration.get_test_component()
        result.feature = self._configuration.get_features_under_test()
        result.skip_reason = self._configuration.skip_reason() if self._configuration.skip() else ''
        result.fail_reason = self._failreason
        result.logpath = os.path.abspath(LogManager.get_base_dir())
        result.logfiles = LogManager.get_logfiles()
        result.retcode = self.retcode
        result.set_dutinformation(self._resources.dutinformations)
        # pylint: disable=unused-variable
        for platform, serialnumber in zip(self._resources.get_platforms(),
                                          self._resources.get_serialnumbers()):
            #  Zipping done to keep platforms and serial numbers aligned in case some sn:s are
            #  missing
            result.dut_vendor.append('')
            result.dut_resource_id.append(serialnumber)
        result.dut_count = self._resources.get_dut_count()
        result.duts = self._resources.resource_configuration.get_dut_configuration()
        if self._resources.resource_configuration.count_hardware() > 0:
            result.dut_type = 'hw'
        elif self._resources.resource_configuration.count_process() > 0:
            result.dut_type = 'process'
        else:
            result.dut_type = None

        self._result_list.append(result)



# pylint: disable=missing-docstring,protected-access,unused-argument,no-self-use
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import unittest
import mock

from icetea_lib.TestBench.Resources import ResourceFunctions
from icetea_lib.ResourceProvider.ResourceConfig import ResourceConfig

TEST_REQS = {
    "requirements": {
        "duts": {
            "*": {"count": 2, "type": "process", "expires": 2000},
            "1": {"nick": "dut1"},
            "2": {"nick": "test_device"}
        }
    }
}


class MockArgs(object):  # pylint: disable=too-few-public-methods
    def __init__(self):
        self.silent = True


class MockLogger(object):
    def __init__(self):
        pass

    def debug(self, *args, **kwargs):
        pass

    def warning(self, *args, **kwargs):
        pass

    def error(self, *args, **kwargs):
        pass


class ResourceMixerTests(unittest.TestCase):

    def _create_dut(self):
        mocked_dut = mock.MagicMock()
        mocked_dut.close_dut = mock.MagicMock(side_effect=[True, KeyboardInterrupt])
        mocked_dut.close_connection = mock.MagicMock()
        mocked_dut.finished = mock.MagicMock(side_effect=[False, True])
        return mocked_dut

    @mock.patch("icetea_lib.TestBench.Resources.time")
    def test_dut_release(self, mock_time):
        mocked_dut = self._create_dut()
        resmixer = ResourceFunctions(mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        resmixer.duts = [mocked_dut]
        resmixer._args = MockArgs()
        resmixer._logger = MockLogger()
        resmixer._resource_provider = mock.MagicMock()
        resmixer._call_exception = mock.MagicMock()
        resmixer.is_my_dut_index = mock.MagicMock(return_value=True)
        resmixer.duts_release()
        mocked_dut.close_dut.assert_called_once()
        mocked_dut.close_connection.assert_called_once()
        resmixer.duts_release()
        self.assertEqual(len(resmixer.duts), 0)

    @mock.patch("icetea_lib.TestBench.Resources.time")
    def test_dut_release_exception(self, mock_time):
        mocked_dut = self._create_dut()
        mocked_dut.close_connection = mock.MagicMock(side_effect=[Exception])
        resmixer = ResourceFunctions(mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        resmixer.duts = [mocked_dut]
        resmixer._args = MockArgs()
        resmixer._logger = MockLogger()
        resmixer._resource_provider = mock.MagicMock()
        resmixer._resource_provider.allocator = mock.MagicMock()
        type(resmixer._resource_provider.allocator).share_allocations = mock.PropertyMock(
            return_value=False)
        resmixer._resource_provider.allocator.release = mock.MagicMock()
        resmixer._call_exception = mock.MagicMock()
        resmixer.is_my_dut_index = mock.MagicMock(return_value=True)
        resmixer.duts_release()
        mocked_dut.close_dut.assert_called_once()
        mocked_dut.close_connection.assert_called_once()
        resmixer._resource_provider.allocator.release.assert_called_once()

    def test_dut_count(self):
        resmixer = ResourceFunctions(mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        resmixer.resource_configuration.count_duts = mock.MagicMock(return_value=1)
        self.assertEqual(resmixer.dut_count(), 1)
        resmixer.resource_configuration = None
        self.assertEqual(resmixer.dut_count(), 0)
        self.assertEqual(resmixer.get_dut_count(), 0)
        resmixer.resource_configuration = ResourceConfig()
        resmixer.resource_configuration.count_duts = mock.MagicMock(return_value=1)
        self.assertEqual(resmixer.get_dut_count(), 1)

    def test_is_allowed_dut_index(self):
        resmixer = ResourceFunctions(mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        resmixer.resource_configuration = ResourceConfig()
        resmixer.resource_configuration.count_duts = mock.MagicMock(return_value=4)
        self.assertTrue(resmixer.is_allowed_dut_index(2))
        self.assertTrue(resmixer.is_allowed_dut_index(4))
        self.assertFalse(resmixer.is_allowed_dut_index(5))

    def test_get_dut(self):
        resmixer = ResourceFunctions(mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        resmixer._logger = MockLogger()
        mocked_dut = self._create_dut()
        resmixer.duts = [mocked_dut]
        self.assertEqual(resmixer.get_dut(1), mocked_dut)
        with self.assertRaises(ValueError):
            resmixer.get_dut(2)

    def test_get_dut_nick(self):
        resmixer = ResourceFunctions(mock.MagicMock(), mock.MagicMock(), mock.MagicMock())
        type(resmixer._configuration).config = mock.PropertyMock(return_value=TEST_REQS)
        resmixer.resource_configuration = ResourceConfig(TEST_REQS)
        resmixer.resource_configuration.resolve_configuration(None)
        self.assertEqual(resmixer.get_dut_nick(1), "dut1")
        self.assertEqual(resmixer.get_dut_nick(2), "test_device")
        self.assertEqual(resmixer.get_dut_nick("1"), "dut1")
        self.assertEqual(resmixer.get_dut_nick("2"), "test_device")
        self.assertEqual(resmixer.get_dut_nick("3"), "3")

    def test_get_dut_index(self):
        resmixer = ResourceFunctions(mock.MagicMock(), mock.MagicMock(), mock.PropertyMock(
            return_value=TEST_REQS))
        resmixer.resource_configuration = ResourceConfig(TEST_REQS)
        resmixer.resource_configuration.resolve_configuration(None)
        self.assertEqual(resmixer.get_dut_index("dut1"), 1)
        self.assertEqual(resmixer.get_dut_index("test_device"), 2)
        with self.assertRaises(ValueError):
            resmixer.get_dut_index("3")

    def test_config_validation_bin_not_defined(self):  # pylint: disable=invalid-name
        duts_cfg = [{}]
        mocked_args = mock.MagicMock()
        type(mocked_args).skip_flash = mock.PropertyMock(return_value=False)
        resources = ResourceFunctions(mocked_args,
                                      mock.MagicMock(), mock.PropertyMock(return_value=TEST_REQS))
        self.assertEqual(resources.validate_dut_configs(duts_cfg, MockLogger()), None)

    def test_config_validation_no_bin(self):
        duts_cfg = [{"application": {"bin": "not.exist"}}]
        mocked_args = mock.MagicMock()
        type(mocked_args).skip_flash = mock.PropertyMock(return_value=False)
        resources = ResourceFunctions(mocked_args, mock.MagicMock(), mock.PropertyMock(
            return_value=TEST_REQS))
        with self.assertRaises(EnvironmentError):
            resources.validate_dut_configs(duts_cfg, MockLogger())

    def test_config_validation_bin_defined(self):  # pylint: disable=invalid-name
        duts_cfg = [{"application": {"bin": __file__}}]
        mocked_args = mock.MagicMock()
        type(mocked_args).skip_flash = mock.PropertyMock(return_value=False)
        resources = ResourceFunctions(mocked_args,
                                      mock.MagicMock(), mock.PropertyMock(return_value=TEST_REQS))
        self.assertEqual(resources.validate_dut_configs(duts_cfg, MockLogger()), None)

    def test_config_validation_no_bin_skip_flash(self):  # pylint: disable=invalid-name
        duts_cfg = [{"application": {"bin": "not.exist"}}]
        mocked_args = mock.MagicMock()
        type(mocked_args).skip_flash = mock.PropertyMock(return_value=True)
        resources = ResourceFunctions(mocked_args,
                                      mock.MagicMock(), mock.PropertyMock(return_value=TEST_REQS))
        self.assertEqual(resources.validate_dut_configs(duts_cfg, MockLogger()), None)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import traceback
import os
import sys
from copy import copy
from collections import Counter

from icetea_lib.TestSuite.TestcaseContainer import TestcaseContainer, DummyContainer


class TestcaseList(object):
    """
    TestcaseList object, a list-like structure with helpers to collect local test cases and
    filter found tests for running.
    """
    def __init__(self, logger=None, testcases=None):
        self.logger = logger
        self._testcases = testcases if testcases else []
        self.search_errors = []
        if self.logger is None:
            import logging
            self.logger = logging.getLogger("TestcaseList")
            if not self.logger.handlers:
                self.logger.addHandler(logging.StreamHandler())
                self.logger.setLevel(logging.INFO)

    def __iter__(self):
        return iter(self._testcases)

    def __len__(self):
        return len(self._testcases)

    def get_list(self):
        """
        Returns the internal list of TestcaseContainers
        """
        return self._testcases

    def get_names(self):
        """
        Gets names of test cases in this TestcaseList.

        :return: list
        """
        lst = []
        for testcase in self._testcases:
            lst.append(testcase.tcname)
        return lst

    def filter(self, filt, tc_names=None):
        """
        Filter test cases from this list into a new TestcaseList object.
        :param filt: TestcaseFilter
        :param tc_names: List of test case names. Is used when running with tc or with a suite
        :return: new TestcaseList with filtered test cases.
        """
        templist = TestcaseList(self.logger)
        if tc_names is not None:
            self._filter_from_names(tc_names, templist, filt)
        else:
            for i, testcase in enumerate(self._testcases):
                if filt.match(testcase, i):
                    templist.append(testcase)

        # Check that all named testcases were found. Append dummies if some are missing
        self._check_filtered_tcs(filt, templist)
        templist.search_errors = self.search_errors
        return templist

    def import_from_path(self, path="./testcases"):
        """
        Import test cases from path to this TestcaseList

        :param path: Path to import from
        :return: Nothing
        """
        local_testcases = self._get_local_testcases(path)
        self._testcases = self._parse_local_testcases(local_testcases, True)

    def get(self, index):
        """
        dict-like getter based on index.

        :param index: Index of test case
        :return: TestcaseContainer or None if index is outside len
        """
        return self._testcases[index] if index < len(self) else None

    def append(self, val):
        """
        Append val to internal list of test cases.

        :param val: test case to append
        :return: Nothing
        """
        self._testcases.append(val)

    def _get_local_testcases(self, tcpath):
        """
        Crawl given path for .py files
        """
        i = 0
        returnlist = []
        if not isinstance(tcpath, str):
            self.logger.error("Error: testcase path is not string")
            sys.exit(0)

        # path is absolute
        tcpath = os.path.abspath(tcpath)
        if len(tcpath.split(os.sep)) > 1:
            strip_dir = os.sep.join(tcpath.split(os.sep)[:-1]) + os.sep
        else:
            strip_dir = ''

        for root, _, files in os.walk(tcpath):
            for file_handle in sorted(files):
                basename, extension = os.path.splitext(file_handle)
                if (basename == '__init__') or extension != '.py':
                    continue
                moduleroot = ''
                modulename = ''
                moduleroot = root.replace(strip_dir, '', 1)
                modulename = moduleroot.replace(os.sep, ".") + '.' + basename
                returnlist.append((modulename, moduleroot, root + os.sep + file_handle))
                i += 1
        if i == 0:
            self.logger.error("Error: No files found in given path: %s", tcpath)
        return returnlist

    def _parse_local_testcases(self, tc_list, verbose):
        """
        Parse list produced by get_local_testcases()
        """
        return_list = []
        if not isinstance(tc_list, list):
            self.logger.error("Error, parseLocalTestcases: Given argument not a list.")
            return return_list
        i = 0
        from icetea_lib.IceteaManager import TCMetaSchema
        schema = TCMetaSchema().get_meta_schema()
        for testcase in tc_list:
            i += 1
            try:
                parsedcases = TestcaseContainer.find_testcases(modulename=testcase[0],
                                                               moduleroot=testcase[1],
                                                               path=testcase[2],
                                                               tc_meta_schema=schema,
                                                               logger=self.logger)
                return_list.extend(parsedcases)
            except (IndexError, TypeError, ValueError):
                self.logger.error("Error, parse_local_testcases: Malformed list item. "
                                  "Skipping item %d", i)
                self.logger.debug("tc: %s", str(testcase))
                if verbose:
                    traceback.print_exc()
            except ImportError as error:
                error_item = {"module": testcase[0], "error": error}
                self.search_errors.append(error_item)
                continue
        return return_list

    def _filter_from_names(self, tc_names, tclist, filt):
        """
        Fill out missing test cases with dummies if a names test case is missing.

        :param tc_names: List of test case names requested.
        :param tclist: List of found test cases.
        :param filt: TestcaseFilter.
        :return: Nothing, modifies tclist in place.
        """
        for tcname in tc_names:
            found = False
            for i, testcase in enumerate(self._testcases):
                if filt.match(testcase, i) and testcase.tcname == tcname:
                    if testcase in tclist:
                        tclist.append(copy(testcase))
                    else:
                        tclist.append(testcase)
                    found = True
            if not found:
                dummy = DummyContainer(self.logger)
                dummy.tcname = tcname
                dummy.set_result_verdict("Test case not found")
                tclist.append(dummy)

    def _check_filtered_tcs(self, filt, tclist):
        """
        Check filtered test cases afor a few special circumstances.

        :param filt: TestcaseFilter.
        :param tclist: List of test cases.
        :return: Nothing, modifies tclist in place.
        """
        if filt.get_filter().get("name") is not False and filt.get_filter().get("name") != "all":
            # If --tc filter is set, length can be max 1
            if len(tclist) != 1:
                dummy = DummyContainer(self.logger)
                dummy.tcname = filt.get_filter().get("name")
                dummy.set_result_verdict("Test case not found")
                tclist.append(dummy)

        if filt.get_filter().get("list") is not False:
            if len(tclist) < len(filt.get_filter().get("list")):
                needed_names = filt.get_filter().get("list")
                found_names = tclist.get_names()
                counter1 = Counter(needed_names)
                counter2 = Counter(found_names)
                diff = counter1 - counter2
                lst = list(diff.elements())

                for name in lst:
                    dummy = DummyContainer(self.logger)
                    dummy.tcname = name
                    dummy.set_result_verdict("Test case not found")
                    tclist.append(dummy)



# pylint: disable=no-member
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Helpers for handling plugins.
"""

from pydash import get
from jsonmerge import merge

from icetea_lib.CliResponseParser import ParserManager
from icetea_lib.Plugin.PluginManager import PluginManager
from icetea_lib.Plugin.PluginManager import PluginException
from icetea_lib.tools.GenericProcess import GenericProcess


class Plugins(object):
    """
    This Mixer manage Test used Plugins.
    """
    def __init__(self, logger, env, args, config):
        super(Plugins, self).__init__()
        self._parser_manager = None
        self._pluginmanager = None
        self._logger = logger
        self._env = env
        self._args = args
        self._config = config

    def init(self, benchapi, logger=None):
        """
        Initialize Parser and Plugin managers.

        :return: Nothing
        """
        self._env = benchapi.env
        if logger:
            self._logger = logger
        self._parser_manager = ParserManager(self._logger)
        self._pluginmanager = PluginManager(responseparser=self._parser_manager,
                                            bench=benchapi,
                                            logger=self._logger)

    def load_plugins(self):
        """
        Initialize PluginManager and Load bench related plugins.

        :return: Nothing
        """
        self._pluginmanager.load_default_tc_plugins()
        self._pluginmanager.load_custom_tc_plugins(self._args.plugin_path)

    @property
    def pluginmanager(self):
        """
        Getter for PluginManager.

        :return: PluginManager
        """
        return self._pluginmanager

    @pluginmanager.setter
    def pluginmanager(self, value):
        """
        Setter for PluginManager.
        """
        self._pluginmanager = value

    # All required external services starting here
    def start_external_services(self):
        """
        Start ExtApps required by test case.

        :return: Nothing
        """
        apps = get(self._config, 'requirements.external.apps', [])
        for app in apps:
            # Check if we have an environment configuration for this app
            conf = app
            try:
                conf = merge(conf, self._env["extApps"][app["name"]])
            except KeyError:
                self._logger.warning("Unable to merge configuration for app %s", app,
                                     exc_info=True if not self._args.silent else False)

            if 'name' in app:
                try:
                    self.pluginmanager.start_external_service(app['name'], conf=conf)
                except PluginException:
                    self._logger.error("Failed to start requested external services.")
                    raise EnvironmentError("Failed to start requested external services.")
                self._logger.info("done")
            else:
                conf_path = None
                conf_cmd = None
                try:
                    conf_path = conf["path"]
                except KeyError:
                    self._logger.warning("No path defined for app %s", app)
                try:
                    conf_cmd = conf["cmd"]
                except KeyError:
                    self._logger.warning("No command defined for app %s", app)
                appname = 'generic'
                newapp = GenericProcess(name=appname, path=conf_path, cmd=conf_cmd)
                newapp.ignore_return_code = True
                newapp.start_process()

    def stop_external_services(self):
        """
        Stop external services started via PluginManager
        """
        self._logger.debug("Stop external services if any")
        self.pluginmanager.stop_external_services()

    def parse_response(self, cmd, response):
        """
        Parse a response for command cmd.

        :param cmd: Command
        :param response: Response
        :return: Parsed response (usually dict)
        """
        return self._parser_manager.parse(cmd, response)



# pylint: disable=no-member,attribute-defined-outside-init
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Main test bench module. Contains the Bench class that subclasses a multitude of mixins.
"""

from icetea_lib.TestBench.BenchApi import BenchApi
from icetea_lib.TestBench.RunnerSM import RunnerSM
from icetea_lib.ReturnCodes import ReturnCodes
from icetea_lib.TestStepError import TestStepError, InconclusiveError


class Bench(BenchApi):
    """
    This is base Bench class which merge all subclasses together to one logical blob.
    Each subclasses (called Mixer) brings some functionality to Bench.
    Logger mixer brings self.logger -instance which are most widely used.
    RunnerMixer manage test execution in right order, like setup_bench, and teardown_bench -calling.
    See rest of Mixer functionality from theirs class descriptions.
    This class brings very top level API's, e.g. constructor
    for whole system and run -method which is called by TestManager when test begins.
    """

    def __init__(self, **kwargs):
        super(Bench, self).__init__(**kwargs)
        self.runner = None

    def run(self):
        """
        Run the test bench.

        :return: int (return code)
        """
        try:
            self._init()
            self.runner = RunnerSM(self, self.logger)
        except (TestStepError, InconclusiveError) as error:
            self.set_failure(ReturnCodes.RETCODE_FAIL_INCONCLUSIVE, str(error))
            self.logger.error(error)
            self.logger.info("Test case verdict: INCONCLUSIVE")
            return ReturnCodes.RETCODE_FAIL_INCONCLUSIVE
        skip = self.runner.check_skip()
        if skip is not None:
            return skip
        retval = self.runner.run()
        if retval in ReturnCodes.INCONCLUSIVE_RETCODES:
            verdict = "INCONCLUSIVE"
        elif retval == ReturnCodes.RETCODE_SKIP:
            verdict = "SKIP"
        elif retval == ReturnCodes.RETCODE_SUCCESS:
            verdict = "PASS"
        else:
            verdict = "FAIL"
        self.logger.info("Test case verdict: %s", verdict)
        return retval



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Deprecated decorator
"""

import inspect
import traceback
import warnings
import functools


def deprecated(message=""):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used first time and filter is set for show DeprecationWarning.
    """
    def decorator_wrapper(func):
        """
        Generate decorator wrapper function
        :param func: function to be decorated
        :return: wrapper
        """
        @functools.wraps(func)
        def function_wrapper(*args, **kwargs):
            """
            Wrapper which recognize deprecated line from source code
            :param args: args for actual function
            :param kwargs: kwargs for actual functions
            :return: something that actual function might returns
            """
            current_call_source = '|'.join(traceback.format_stack(inspect.currentframe()))
            if current_call_source not in function_wrapper.last_call_source:
                warnings.warn("Function {} is now deprecated! {}".format(func.__name__, message),
                              category=DeprecationWarning, stacklevel=2)
                function_wrapper.last_call_source.add(current_call_source)

            return func(*args, **kwargs)

        function_wrapper.last_call_source = set()

        return function_wrapper
    return decorator_wrapper


# Print deprecated messages always. This might causes side effect
# where deprecation warnings from another dependencies are also printed but
# maybe it's not bad thing eventually.
warnings.simplefilter('always', DeprecationWarning)  # turn off filter



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

ResourceRequirements module. Deals with requirements for resources that are to be allocated for
test cases. Handles storage of requirements and processing requirements to formats required by
allocators.
"""

from jsonmerge import merge
from icetea_lib.tools.tools import recursive_dictionary_get as recursive_search


class ResourceRequirements(object):
    """
    ResourceRequirements class. Contains methods for getting and setting requirement values as
    well as processing requirements into formats supported by allocators.
    """
    def __init__(self, requirement_dict=None):
        self._requirements = requirement_dict if requirement_dict else {}

    def set(self, key, value):
        """
        Sets the value for a specific requirement.

        :param key: Name of requirement to be set
        :param value: Value to set for requirement key
        :return: Nothing, modifies requirement
        """
        if key == "tags":
            self._set_tag(tags=value)
        else:
            if isinstance(value, dict) and key in self._requirements and isinstance(
                    self._requirements[key], dict):
                self._requirements[key] = merge(self._requirements[key], value)
            else:
                self._requirements[key] = value

    def get(self, key):
        """
        Gets contents of requirement key.
        Switches to recursive search if dots ('.') are found in the key.

        :param key: key or dot separated string of keys to look for.
        :return: contents of requirement key/results of search or None.
        """
        # Catch the case where the key is "."
        if "." in key and len(key) > 1:
            return self._recursive_get(key)
        return self._requirements.get(key, None)

    def __getitem__(self, item):
        return self._requirements[item]

    def _recursive_get(self, key, dic=None):
        """
        Gets contents of requirement key recursively so users can search for
        specific keys inside nested requirement dicts.

        :param key: key or dot separated string of keys to look for.
        :param dic: Optional dictionary to use in the search.
        If not provided, self._requirements is used.
        :return: results of search or None
        """
        return recursive_search(key, dic) if dic else recursive_search(key, self._requirements)

    def get_requirements(self):
        """
        Return requirements as dict.

        :return: Dictionary
        """
        return self._requirements

    def _set_tag(self, tag=None, tags=None, value=True):
        """
        Sets the value of a specific tag or merges existing tags with a dict of new tags.
        Either tag or tags must be None.

        :param tag: Tag which needs to be set.
        :param tags: Set of tags which needs to be merged with existing tags.
        :param value: Value to set for net tag named by :param tag.
        :return: Nothing
        """
        existing_tags = self._requirements.get("tags")
        if tags and not tag:
            existing_tags = merge(existing_tags, tags)
            self._requirements["tags"] = existing_tags
        elif tag and not tags:
            existing_tags[tag] = value
            self._requirements["tags"] = existing_tags

    def remove_empty_tags(self, tags=None):
        """
        Tags whose value is set to None shall be removed from tags.
        :param tags: Tags which are to be processed.
        If None, tags found in self._requirements are used.
        :return: If tags is not None, returns dict with processed tags. Else returns None.
        """
        new_tags = {}
        old_tags = tags if tags else self.get("tags")
        for tag in old_tags.keys():
            if old_tags[tag] is not None:
                new_tags[tag] = old_tags[tag]
        if not tags:
            self._requirements["tags"] = new_tags
            return None
        return new_tags



# pylint: disable=missing-docstring,protected-access

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest

from icetea_lib.ResourceProvider.ResourceRequirements import ResourceRequirements


class ResourceRequirementTestcase(unittest.TestCase):

    def setUp(self):
        self.simple_testreqs = {
            "type": "process",
            "allowed_platforms": [],
            "expires": 2000,
            "nick": None,
            "tags": {"test": True}
        }
        self.simple_testreqs2 = {
            "type": "process",
            "allowed_platforms": ["DEV3"],
            "nick": None,
        }
        self.recursion_testreqs = {
            "type": "process",
            "allowed_platforms": ["DEV3"],
            "application": {"bin": "test_binary"},
            "nick": None,
        }

        self.actual_descriptor1 = {"platform_name": "DEV2", "resource_type": "mbed"}
        self.actual_descriptor2 = {"platform_name": "DEV1", "resource_type": "process"}
        self.actual_descriptor3 = {"platform_name": "DEV3", "resource_type": "process"}
        self.actual_descriptor4 = {"resource_type": "process", "bin": "test_binary"}

    def test_get(self):
        dutreq = ResourceRequirements(self.simple_testreqs)
        self.assertEqual(dutreq.get("type"), "process")

        dutreq = ResourceRequirements(self.recursion_testreqs)
        self.assertEqual(dutreq.get("application.bin"), "test_binary")
        self.assertIsNone(dutreq.get("application.bin.not_exist"))

    def test_set(self):
        dutreq = ResourceRequirements(self.simple_testreqs)
        dutreq.set("test_key", "test_val")
        self.assertEqual(dutreq._requirements["test_key"], "test_val")
        # Test override
        dutreq.set("test_key", "test_val2")
        self.assertEqual(dutreq._requirements["test_key"], "test_val2")

        # test tags merging. Also a test for set_tag(tags=stuff)
        dutreq.set("tags", {"test": False, "test2": True})
        self.assertEqual(dutreq._requirements["tags"], {"test": False, "test2": True})
        dutreq.set("tags", {"test2": False})
        self.assertEqual(dutreq._requirements["tags"], {"test": False, "test2": False})

    def test_set_tags(self):
        dutreq = ResourceRequirements(self.simple_testreqs)
        dutreq._set_tag(tag="test", value=False)
        dutreq._set_tag(tag="test2", value=True)
        self.assertDictEqual(dutreq._requirements["tags"], {"test": False, "test2": True})

    def test_empty_tags(self):
        dutreq = ResourceRequirements(self.simple_testreqs)
        dutreq._set_tag("test", value=None)
        dutreq.remove_empty_tags()
        self.assertEqual(dutreq._requirements["tags"], {})
        self.assertEqual(dutreq.remove_empty_tags(tags={"test1": True, "test2": None}),
                         {"test1": True})


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

DutTcp module. This is a prototype of a Dut-over-tcp implementation and is not used.
"""

import socket
from icetea_lib.tools import strip_escape
from icetea_lib.tools import num
from icetea_lib.DeviceConnectors.Dut import Dut


class DutTcp(Dut):
    '''
    Draft version of TCP -dut type
    '''
    def __init__(self, name='tcp'):
        Dut.__init__(self, name=name)
        self.port = None
        self.type = 'socket'

    def open_connection(self):
        """
        Open connection over TCP socket.
        :return: Nothing
        """
        self.logger.debug("Open COM %s", self.comport)
        self.port = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        (ip_addr, port) = self.comport.split(':')
        self.port.connect(ip_addr, num(port))

    def close_connection(self):
        """
        Close TCP port
        :return: Nothing
        """
        if self.port:
            self.port.close()
            self.logger.debug("Close TCP port")

    def writeline(self, data):
        """
        Write data to port
        :param data: data to write
        :return: Nothing
        """
        self.port.send(data)

    def readline(self, timeout=None):  # timeout is not in use
        """
        Read data from port and strip escape characters
        :param timeout:
        :return: Stripped line.
        """
        fil = self.port.makefile()
        line = fil.readline()
        return strip_escape(line.strip())

    def print_info(self):
        pass

    def _flash_needed(self, **kwargs):
        pass

    def get_info(self):
        pass

    def reset(self, method=None):
        pass



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
# pylint: disable=missing-docstring,invalid-name,anomalous-backslash-in-string

import sys
import unittest
from icetea_lib.wireshark import NwPacket
from icetea_lib.wireshark import NwPacketManager
from icetea_lib.TestStepError import TestStepError
sys.path.append('.')

if sys.version_info[0] == 2:  # Python2
    import mock
else:                         # Python3
    from unittest import mock  # pylint: disable=no-name-in-module



class TestVerify(unittest.TestCase):

    def test_packetparser(self):
        string = open('test/data/BeaconRequest.txt', 'r').read()
        packet = NwPacket(string)
        self.assertTrue(NwPacket.verify(  # Successful
            packet,
            {"WPAN": {
                "Command Identifier": "Beacon Request",
                "Destination Addressing Mode": "Short/16-bit",
                "Destination PAN": "0xffff",
                "Frame Version": "0"
            }}
        ))
        self.assertFalse(NwPacket.verify(  # Unsuccessful
            packet,
            {"WPAN": {
                "Command Identifier": "Bacon Request",
                "Destination Addressing Moda": "Short/16-bit",
                "Destination PAN": "0xffff"
            }}
        ))

    def test_packetparser6lowpan(self):
        string = open('test/data/6lowpanping.txt', 'r').read()
        p = NwPacket(string)
        self.assertTrue(NwPacket.verify(  # Successful
            p,
            {"6LOWPAN": {
                "Destination": "fe80::166e:a00:0:2",
                "Source address compression": "Stateless",
                "Context identifier extension": "False"
            }, "ICMPV6":{
                "Response To": "17"
            }}
        ))

    def test_packetparser6lowpan2(self):
        string = open('test/data/6lowpanping2.txt', 'r').read()
        packet = NwPacket(string)
        self.assertTrue(NwPacket.verify(  # Successful
            packet,
            {"6LOWPAN": {
                "Destination": "fe80::166e:a00:0:2",
                "Source address compression": "Stateless",
                "Context identifier extension": "False"
            }, "UDP": {
                "Source Port": "65534"
            }, "IPV6": {
                "Source": "fe80::ff:fe00:0"
            }}
        ))

    def test_packetparser6lowpan3(self):
        string = open('test/data/6lowpanudp.txt', 'r').read()
        packet = NwPacket(string)
        self.assertTrue(
            NwPacket.verify(
                packet,
                {
                    "6LOWPAN": {
                        "Destination": "fd00:db8::ff:fe00:face"
                    },
                    "IPV6": {
                        "RPLInstanceID": "0x82"
                    },
                    "UDP": {
                        "Destination Port": "7"
                    }
                }
            )
        )

    def test_packetparserfullrow(self):
        string = open('test/data/rpl_dio.txt', 'r').read()
        packet = NwPacket(string)
        self.assertTrue(NwPacket.verify(
            packet,
            {
                "IPV6": {
                    "Source": "fe80::ff:fe00:face",
                    "Destination": "ff02::1a"
                },
                "ICMPV6": {
                    "Code": "1",
                    "*1": "ICMPv6 RPL Option \(Routing Information fd00:db8::/64\)",
                    "*2": "ICMPv6 RPL Option \(Prefix Information fd00:db8::ff:fe00:face/64\)",
                    "*3": "ICMPv6 RPL Option \(Routing Information ::/0\)"
                }
            }
        ))

    def test_packetparserlegacy(self):
        string = open('test/data/BeaconRequest.txt', 'r').read()
        packet = NwPacket(string)
        self.assertTrue(NwPacket.verify(
            packet,
            {
                "WPAN.Command Identifier": "Beacon Request",
                "WPAN.Destination Addressing Mode": "Short/16-bit",
                "WPAN.Destination PAN": "0xffff",
                "WPAN.Frame Version": "0"
            }
        ))
        self.assertFalse(NwPacket.verify(  # Unsuccessful
            packet,
            {
                "WPAN.Command Identifier": "Bacon Request",
                "WPAN.Destination Addressing Moda": "Short/16-bit",
                "WPAN.Destination PAN": "0xffff"
            }
        ))

    def test_packetparsermultilayer(self):
        string = open('test/data/BeaconRequest.txt', 'r').read()
        packet = NwPacket(string)
        self.assertTrue(NwPacket.verify(  # Successful
            packet,
            {"WPAN": {
                "Destination PAN": "0xffff"
            }, "IP": {
                "Protocol": "UDP"
            }}
        ))
        self.assertFalse(NwPacket.verify(  # Unsuccessful
            packet,
            {"WPAN": {
                "Destination PAN": "0xffff"
            }, "IP": {
                "Protocol": "XXX"
            }}
        ))

    def test_packetparserregexvalue(self):
        string = open('test/data/BeaconRequest.txt', 'r').read()
        packet = NwPacket(string)
        self.assertTrue(NwPacket.verify(
            packet,
            {"WPAN":{
                "Destination PAN": "0xfff[abcdef]"
            }, "IP":{
                "Protocol": "KGB|UDP|VH1"
            }}
        ))

    @mock.patch('icetea_lib.LogManager.get_bench_logger')
    def test_packetmangager_happyday(self, loggerpatch):
        loggerpatch.return_value = mock.MagicMock()
        p1 = open('test/data/BeaconRequest.txt', 'r').read()
        p2 = open('test/data/BeaconRequest2.txt', 'r').read()
        manager = NwPacketManager()
        manager.push(p1)
        manager.push(p2)
        self.assertTrue(manager.verifyPackets([{
            "WPAN":{
                "Command Identifier": "Beacon Request",
                "Destination PAN": "0xffff",
                "Sequence Number": "79"
            }
        }]) is None)

    @mock.patch('icetea_lib.LogManager.get_bench_logger')
    def test_packetmangager_sadday(self, loggerpatch):
        loggerpatch.return_value = mock.MagicMock()
        p1 = open('test/data/BeaconRequest.txt', 'r').read()
        p2 = open('test/data/BeaconRequest2.txt', 'r').read()
        manager = NwPacketManager()
        manager.push(p1)
        manager.push(p2)
        with self.assertRaises(TestStepError):
            self.assertTrue(manager.verifyPackets([{
                "WPAN": {
                    "Command Identifiers": "Bacon Request",
                    "Destination PAN": "0xffff"
                }
            }]) is None)
        with self.assertRaises(TestStepError):
            self.assertTrue(manager.verifyPackets([{
                "WPAN": {
                    "Command Identifier": "Beacon Request",
                    "Destination PAN": "0xffff",
                    "Sequence Number": "80"
                }
            }]) is None)

    @mock.patch('icetea_lib.LogManager.get_bench_logger')
    def test_packetcounter6lowpan(self, loggerpatch):
        loggerpatch.return_value = mock.MagicMock()
        p1 = open('test/data/BeaconRequest2.txt', 'r').read()
        p2 = open('test/data/6lowpanping.txt', 'r').read()
        p3 = open('test/data/BeaconRequest.txt', 'r').read()
        p4 = open('test/data/6lowpanping2.txt', 'r').read()
        manager = NwPacketManager()
        manager.push(p1)
        manager.push(p2)
        manager.push(p3)
        manager.push(p4)
        self.assertTrue(manager.countPackets({
            "WPAN": {
                "Extended Source": "14:6e:0a:00:00:00:00:01",
            },
            "IPV6": {
                "Destination": "fe80::166e:a00:0:2"
            }
        }) == 2)

    '''
    # these test needs sniffer HW and valid configurations.
    def test_liveCapture(self):

        wshark = Wireshark()
        wshark.startCapture('wireshark.pcap')
        time.sleep(5)
        wshark.setMark('marker#1')
        time.sleep(1)
        wshark.setMark('marker#2')
        self.assertIsInstance( wshark.stopCapture(), int )

        self.assertEqual( wshark.findIndexByMark('start'), 0 )
        self.assertEqual( wshark.findIndexByMark('unknown'), None )

        self.assertGreaterEqual( wshark.findIndexByMark('marker#1'), wshark.findIndexByMark('marker#2') )

    def test_wireshark(self):
        capture = pyshark.LiveCapture(interface='Sniffer')
        print('Start Live capturing')
        capture.sniff(timeout=10)
        for packet in capture.sniff_continuously(packet_count=50):
            print 'Just arrived:', packetl
        print("Stop Live Capturing")
        print( capture )
        for packet in capture:
            print(packet)
        import netifaces
        print( netifaces.inter() )
        print( capture.interfaces )

        self.assertTrue(True)
        with self.assertRaises(LookupError):
            self.assertTrue(raise LookupError())
    '''  # pylint: disable=pointless-string-statement


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

DutProcess module.
"""

from icetea_lib.DeviceConnectors.Dut import Dut, DutConnectionError
from icetea_lib.tools.GenericProcess import GenericProcess
from icetea_lib.DeviceConnectors.DutInformation import DutInformation
from icetea_lib.build.build import Build
# pylint: disable=redefined-builtin


class DutProcess(Dut, GenericProcess):  # pylint: disable=too-many-instance-attributes
    """
    DutProcess class, subclasses both Dut and GenericProcess. Implements an interface for
    communicating with a process as if it were a device under test.
    """
    def __init__(self, type='process', name='process', config=None, params=None):
        Dut.__init__(self, name=name, params=params)
        GenericProcess.__init__(self, self.name, logger=self.logger)
        self.disable_io_prints()  # because those are printed in Dut object
        self.proc = False
        self.type = type
        self.config = config if config else {}
        self.dutinformation = DutInformation(self.type,
                                             self.resource_id if self.resource_id else "",
                                             index=None, build=None)
        self.command = None

    def open_connection(self):
        """
        Open connection by starting the process.

        :raises: DutConnectionError
        """
        self.logger.debug("Open CLI Process '%s'",
                          (self.comport), extra={'type': '<->'})
        self.cmd = self.comport if isinstance(self.comport, list) else [self.comport]
        if not self.comport:
            raise DutConnectionError("Process not defined!")
        try:
            self.build = Build.init(self.cmd[0])
        except NotImplementedError as error:
            self.logger.error("Build initialization failed. Check your build location.")
            self.logger.debug(error)
            raise DutConnectionError(error)
        # Start process&reader thread. Call Dut.process_dut() when new data is coming
        app = self.config.get("application")
        if app and app.get("bin_args"):
            self.cmd = self.cmd + app.get("bin_args")
        try:
            self.start_process(self.cmd, processing_callback=lambda: Dut.process_dut(self))
        except KeyboardInterrupt:
            raise
        except Exception as error:
            raise DutConnectionError("Couldn't start DUT target process {}".format(error))

    def prepareConnectionClose(self):  # pylint: disable=C0103
        """
        Deprecated version of prepare_connection_close. Still present for backwards compatibility.

        :return: Nothing
        """
        self.logger.warning("prepareConnectionClose deprecated, use prepare_connection_close")
        self.prepare_connection_close()

    def prepare_connection_close(self):
        """
        exit the process if it is alive.

        :return: Nothing
        """
        pass

    def close_connection(self):
        """
        Stop the process.

        :return: Nothing
        """
        self.logger.debug("Close CLI Process '%s'" % self.cmd, extra={'type': '<->'})
        self.stop_process()

    def writeline(self, data, crlf="\n"):  # pylint: disable=arguments-differ
        """
        Write data to process.

        :param data: data to write
        :param crlf: line end character
        :return: Nothing
        """
        GenericProcess.writeline(self, data, crlf=crlf)

    def readline(self, timeout=1):
        """
        Read a line from the process.

        :param timeout: Timeout
        :return: read line
        """
        return GenericProcess.readline(self, timeout=timeout)

    def reset(self, method=None):
        """
        Not implemented
        """
        self.logger.info("Reset not implemented for process DUT")

    def print_info(self):
        """
        Print information of this dut.

        :return: Nothing.
        """
        info_string = "DutProcess {}, \n".format(self.name)
        if self.comport:
            info_string = info_string + "CMD {} \n".format(self.comport)
        if self.location:
            info_string = info_string + "Location: x = {}, y = {} \n".format(self.location.x_coord,
                                                                             self.location.y_coord)
        if self.config:
            info_string = info_string + "Configuration for this DUT:\n {} \n".format(self.config)
        self.logger.info(info_string)

    def get_info(self):
        """
        Get DutInformation object of this dut.

        :return: DutInformation
        """
        return self.dutinformation

    def get_config(self):
        """
        Get configuration of this dut.

        :return: dictionary
        """
        return self.config

    def _flash_needed(self, **kwargs):
        pass



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

DutConsole module, contains DutConsole class which inherits DutProcess.
"""


# Disable "string statement has no effect" warning
# pylint: disable=W0105


from icetea_lib.Plugin.plugins.LocalAllocator.DutProcess import DutProcess


class DutConsole(DutProcess):
    """
        Configuration for DutConsole. The supported configuration parameters are:
        "type":             The type of console connection; possible values: "SSH" (default)
        "username":         The username that should be used for login; default is 'arm'
        "hostname":         The hostname or IP address of the remote host; default is 'localhost'
        "port":             Port for the console connection; default is 22
        "app":              The name of the executable that creates the connection;
            default is '/usr/bin/ssh'
        "cwd":              The directory where the executable is run;
            default is None (current directory)
        "args":             Extra arguments for the executable; default is '-tt'
        "shell":            The type of shell (if any) used at the remote host; default is 'bash'
    """

    conf = {
        "type": "SSH",
        "username": "arm",
        "hostname": "localhost",
        "app": "/usr/bin/ssh",
        "port": "22",
        "cwd": None,
        "args": "-tt",
        "shell": "bash"
    }

    def __init__(self, name, conf=None, params=None):
        DutProcess.__init__(self, name=name, params=None)
        self.config.update(self.conf)

        if conf is not None:
            self.config.update(conf)

        # Set up SSH connection
        if self.config["type"] == "SSH":
            self.cmd = self.config["app"] + " " + \
                       self.config["args"] + " " + \
                       self.config["username"] + "@" + \
                       self.config["hostname"] + " -p " + \
                       self.config["port"]
            self.path = self.config["cwd"]

        self.ignore_return_code = True
        self.comport = self.cmd

    def init_cli(self):
        # Set up the Bash Unix shell
        if self.config["shell"] == "bash":
            self.execute_command("export PROMPT_COMMAND='RC=$?;echo \"retcode: $RC\";'")

    def prepareConnectionClose(self): #pylint: disable=C0103
        """
        Deprecated version of prepare_connection_close. Still present for backwards compatibility

        :return: Nothing
        """
        self.logger.warning("prepareConnectionClose deprecated, use prepare_connection_close")
        self.prepare_connection_close()

    def prepare_connection_close(self):
        # No actions...
        return

    def reset(self, method=None):
        # No actions...
        return

    def writeline(self, data, crlf="\n"):
        DutProcess.writeline(self, data, crlf)

    def print_info(self):
        info_string = "DutConsole {} \n".format(self.name)
        if self.config:
            info_string = info_string + "Configuration for this DUT:\n {} \n".format(self.config)
        if self.comport:
            info_string = info_string + "COM port: {} \n".format(self.comport)
        if self.location:
            info_string = info_string + "Location: x = {}, y = {} \n".format(self.location.x_coord,
                                                                             self.location.y_coord)

        self.logger.info(info_string)

    def get_config(self):
        return self.config

    def _flash_needed(self, **kwargs):
        pass



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

DutSerial module contains the DutSerial Dut subclass and two parameter objects, which contain
serial and chunk mode parameters.
"""

# Disable too many arguments warning and string statement has no effect warning
# Disable Too few public methods warning, too many public methods warning
# pylint: disable=R0913,W0105,R0903,R0904
# pylint: disable=too-many-instance-attributes,unused-argument

import time
from collections import deque
from threading import Thread
from serial import SerialException
from prettytable import PrettyTable

from icetea_lib.DeviceConnectors.Dut import Dut, DutConnectionError
from icetea_lib.enhancedserial import EnhancedSerial
from icetea_lib.tools.tools import strip_escape, split_by_n
from icetea_lib.DeviceConnectors.DutInformation import DutInformation


class SerialParams(object):
    """
    SerialParams object for storing serial connection parameters.
    """
    def __init__(self, timeout=0.01, xonxoff=False, rtscts=False, baudrate=460800):
        self.timeout = timeout
        self.xonxoff = xonxoff
        self.rtscts = rtscts
        self.baudrate = baudrate

    def get_params(self):
        """
        Get parameters as a tuple.

        :return: timeout, xonxoff, rtscts, baudrate
        """
        return self.timeout, self.xonxoff, self.rtscts, self.baudrate


class ChunkModeParams(object):
    """
    ChunkModeParams object for storing chunk mode parameters
    """
    def __init__(self, on=False, size=1, chunk_delay=0.01, start_delay=0):
        self.enabled = on
        self.size = size
        self.chunk_delay = chunk_delay
        self.start_delay = start_delay

    def get_params(self):
        """
        Get parameters.

        :return: enabled, size, chunk_delay, start_delay
        """
        return self.enabled, self.size, self.chunk_delay, self.start_delay


class DutSerial(Dut):
    """
    DutSerial Object. Inherits from Dut object. Represents a local hardware device connected to USB
    """
    def __init__(self, name='serial', port=None, baudrate=460800, config=None,
                 ch_mode_config=None, serial_config=None, params=None):
        Dut.__init__(self, name=name, params=params)
        ch_mode_config = ch_mode_config if ch_mode_config is not None else {}
        serial_config = serial_config if serial_config is not None else {}
        self.readthread = None
        self.port = False
        self.comport = port
        self.type = 'serial'
        self.name = port
        self.platform = ''
        self.serialparams = SerialParams(timeout=serial_config.get("serial_timeout", 0.01),
                                         xonxoff=serial_config.get("serial_xonxoff", False),
                                         rtscts=serial_config.get("serial_rtscts", False),
                                         baudrate=baudrate)

        self.chunkmodeparams = ChunkModeParams(on=ch_mode_config.get("ch_mode", False),
                                               size=ch_mode_config.get("ch_mode_chunk_size", 1),
                                               chunk_delay=ch_mode_config.get("ch_mode_ch_delay",
                                                                              0.01),
                                               start_delay=ch_mode_config.get("ch_mode_start_delay",
                                                                              0))
        self.input_queue = deque()  # Input queue
        self.daemon = True  # Allow Python to stop us
        self.keep_reading = False

        if config:
            self.config.update(config)
            self.device = config.get("allocated", None)
            init_cli_cmds = None
            if "init_cli_cmds" in config["application"]:
                init_cli_cmds = config["application"]["init_cli_cmds"]
            if init_cli_cmds is not None:
                self.set_init_cli_cmds(init_cli_cmds)
            post_cli_cmds = None
            if "post_cli_cmds" in config["application"]:
                post_cli_cmds = config["application"]["post_cli_cmds"]
            if post_cli_cmds is not None:
                self.set_post_cli_cmds(post_cli_cmds)
        tid = self.config.get('allocated', {}).get('target_id', "unknown")
        self.dutinformation = DutInformation("serial",
                                             tid,
                                             index=self.index, build=self.build)

    """Properties"""

    @property
    def ch_mode(self):
        """
        :return: True if chunk mode enabled, False otherwise
        """
        return self.chunkmodeparams.enabled

    @ch_mode.setter
    def ch_mode(self, value):
        self.chunkmodeparams.enabled = value

    @property
    def ch_mode_chunk_size(self):
        """
        :return: Chunk size
        """
        return self.chunkmodeparams.size

    @ch_mode_chunk_size.setter
    def ch_mode_chunk_size(self, value):
        self.chunkmodeparams.size = value

    @property
    def ch_mode_ch_delay(self):
        """
        :return: Chunk delay
        """
        return self.chunkmodeparams.chunk_delay

    @ch_mode_ch_delay.setter
    def ch_mode_ch_delay(self, value):
        self.chunkmodeparams.chunk_delay = value

    @property
    def ch_mode_start_delay(self):
        """
        :return: Chunk start delay
        """
        return self.chunkmodeparams.start_delay

    @ch_mode_start_delay.setter
    def ch_mode_start_delay(self, value):
        self.chunkmodeparams.start_delay = value

    @property
    def serial_baudrate(self):
        """
        Getter for serial baudrate.

        :return: int
        """
        return self.serialparams.baudrate

    @serial_baudrate.setter
    def serial_baudrate(self, value):
        self.serialparams.baudrate = value

    @property
    def serial_timeout(self):
        """
        :return: Serial timeout
        """
        return self.serialparams.timeout

    @serial_timeout.setter
    def serial_timeout(self, value):
        """
        Setter for serial connection timeout.

        :param value: Value to set
        :return: Nothing
        """
        self.serialparams.timeout = value

    @property
    def serial_xonxoff(self):
        """
        :return: xonxoff value as as Boolean
        """
        return self.serialparams.xonxoff

    @serial_xonxoff.setter
    def serial_xonxoff(self, value):
        self.serialparams.xonxoff = value

    @property
    def serial_rtscts(self):
        """
        :return: Rtscts as boolean
        """
        return self.serialparams.rtscts

    @serial_rtscts.setter
    def serial_rtscts(self, value):
        self.serialparams.rtscts = value

    """Methods"""

    def get_resource_id(self):
        """
        Get resource id (target id) from config dictionary.

        :return: target_id or None if not found
        """
        return self.config.get('allocated').get('target_id')

    def flash(self, binary_location=None, forceflash=None):  # pylint: disable=too-many-branches
        """
        Nothing, not implemented.
        """
        self.logger.warning("Flashing is not supported for this dut type.")
        return True

    def get_info(self):
        """
        Get DutInformation object from this Dut.

        :return: DutInformation object
        """
        return self.dutinformation

    # open serial port connection
    def open_connection(self):
        """
        Open serial port connection.

        :return: Nothing
        :raises: DutConnectionError if serial port was already open or a SerialException occurs.
        ValueError if EnhancedSerial __init__ or value setters raise ValueError
        """
        if self.readthread is not None:
            raise DutConnectionError("Trying to open serial port which was already open")

        self.logger.info("Open Connection "
                         "for '%s' using '%s' baudrate: %d" % (self.dut_name,
                                                               self.comport,
                                                               self.serial_baudrate),
                         extra={'type': '<->'})
        if self.serial_xonxoff:
            self.logger.debug("Use software flow control for dut: %s" % self.dut_name)
        if self.serial_rtscts:
            self.logger.debug("Use hardware flow control for dut: %s" % self.dut_name)
        try:
            self.port = EnhancedSerial(self.comport)
            self.port.baudrate = self.serial_baudrate
            self.port.timeout = self.serial_timeout
            self.port.xonxoff = self.serial_xonxoff
            self.port.rtscts = self.serial_rtscts
            self.port.flushInput()
            self.port.flushOutput()
        except SerialException as err:
            self.logger.warning(err)
            raise DutConnectionError(str(err))
        except ValueError as err:
            self.logger.warning(err)
            raise ValueError(str(err))

        if self.ch_mode:
            self.logger.info("Use chunk-mode with size %d, delay: %.3f when write data" %
                             (self.ch_mode_chunk_size, self.ch_mode_ch_delay),
                             extra={'type': '<->'})
            time.sleep(self.ch_mode_start_delay)
        else:
            self.logger.info("Use normal serial write mode", extra={'type': '<->'})
        if self.params.reset:
            self.reset()
        # Start the serial reading thread
        self.readthread = Thread(name=self.name, target=self.run)
        self.readthread.start()

    def prepareConnectionClose(self):  # pylint: disable=C0103
        """
        Deprecated version of prepare_connection_close. Still present for backwards compatibility.

        :return: Nothing
        """
        self.logger.warning("prepareConnectionClose deprecated, use prepare_connection_close")
        self.prepare_connection_close()

    def prepare_connection_close(self):
        """
        Sends post-cli-cmds and stops the read thread.

        :return: Nothing
        """
        try:
            self.init_cli_human()
        except KeyboardInterrupt:
            pass
        self.stop()

    # close serial port connection
    def close_connection(self):  # pylint: disable=C0103
        """
        Closes serial port connection.

        :return: Nothing
        """
        if self.port:
            self.stop()
            self.logger.debug("Close port '%s'" % self.comport,
                              extra={'type': '<->'})
            self.port.close()
            self.port = False

    def reset(self, method=None):
        """
        Resets the serial device. Internally calls __send_break().

        :param method: Not used for DutSerial
        :return: Nothing
        """
        self.logger.info('Reset serial device %s' % self.name)
        self.__send_break()

    def __sendBreak(self):  # pylint: disable=C0103
        """
        Deprecated, present for backwards compatibility.

        :return: result of EnhancedSerial safe_sendBreak()
        """
        self.logger.warning("__send_Break deprecated, use __send_break")
        return self.__send_break()

    def __send_break(self):
        """
        Sends break to device.

        :return: result of EnhancedSerial safe_sendBreak()
        """
        if self.port:
            self.logger.debug("sendBreak to device to reboot", extra={'type': '<->'})
            result = self.port.safe_sendBreak()
            time.sleep(1)
            if result:
                self.logger.debug("reset completed", extra={'type': '<->'})
            else:
                self.logger.warning("reset failed", extra={'type': '<->'})
            return result
        return None

    # transfer data to the serial port
    def writeline(self, data):
        """
        Writes data to serial port.

        :param data: Data to write
        :return: Nothing
        :raises: IOError if SerialException occurs.
        """
        try:
            if self.ch_mode:
                data += "\n"
                parts = split_by_n(data, self.ch_mode_chunk_size)
                for split_str in parts:
                    self.port.write(split_str.encode())
                    time.sleep(self.ch_mode_ch_delay)
            else:
                self.port.write((data + "\n").encode())
        except SerialException as err:
            self.logger.exception("SerialError occured while trying to write data {}.".format(data))
            raise RuntimeError(str(err))

    # read line from serial port
    def _readline(self, timeout=1):
        """
        Read line from serial port.

        :param timeout: timeout, default is 1
        :return: stripped line or None
        """
        line = self.port.readline(timeout=timeout)
        return strip_escape(line.strip()) if line is not None else line

    def peek(self):
        """
        Peek into the port line buffer to see if there are incomplete lines.

        :return: str
        """
        if self.port:
            return self.port.peek()
        return ""

    def run(self):
        """
        Read lines while keep_reading is True. Calls process_dut for each received line.

        :return: Nothing
        """
        self.keep_reading = True
        while self.keep_reading:
            line = self._readline()
            if line:
                self.input_queue.appendleft(line)
                Dut.process_dut(self)

    def stop(self):
        """
        Stops and joins readthread.

        :return: Nothing
        """
        self.keep_reading = False
        if self.readthread is not None:
            self.readthread.join()
            self.readthread = None

    def readline(self, timeout=1):
        """
        Pops from input_queue.

        :param timeout: Not used
        :return: first item in input_queue or None
        """
        try:
            return self.input_queue.pop()
        except IndexError:
            pass
        return None

    def print_info(self):
        """
        Prints Dut information nicely formatted into a table.
        """
        table = PrettyTable()
        start_string = "DutSerial {} \n".format(self.name)
        row = []
        info_string = ""
        if self.config:
            info_string = info_string + "Configuration for this DUT:\n\n {} \n".format(self.config)
        if self.comport:
            table.add_column("COM port", [])
            row.append(self.comport)
        if self.port:
            if hasattr(self.port, "baudrate"):
                table.add_column("Baudrate", [])
                row.append(self.port.baudrate)
            if hasattr(self.port, "xonxoff"):
                table.add_column("XON/XOFF", [])
                row.append(self.port.xonxoff)
            if hasattr(self.port, "timeout"):
                table.add_column("Timeout", [])
                row.append(self.port.timeout)
            if hasattr(self.port, "rtscts"):
                table.add_column("RTSCTS", [])
                row.append(self.port.rtscts)
        if self.location:
            table.add_column("Location", [])
            row.append("X = {}, Y = {}".format(self.location.x_coord, self.location.y_coord))
        self.logger.info(start_string)
        self.logger.debug(info_string)
        table.add_row(row)
        print(table)

    def get_config(self):
        """
        Gets configuration dictionary.

        :return: configuration as a dictionary
        """
        return self.config

    def _flash_needed(self, **kwargs):
        """
        Check if flashing is needed. Flashing can be skipped if resource binary_sha1 attribute
        matches build sha 1 and forceflash is not True.

        :param kwargs: Keyword arguments (forceflash: Boolean)
        :return: Boolean
        """
        return False
